from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..types import UNSET, Unset

T = TypeVar("T", bound="AlignedSequence")


@attr.s(auto_attribs=True, repr=False)
class AlignedSequence:
    """  """

    _bases: Union[Unset, str] = UNSET
    _dna_sequence_id: Union[Unset, None, str] = UNSET
    _name: Union[Unset, str] = UNSET
    _pairwise_identity: Union[Unset, float] = UNSET
    _trim_end: Union[Unset, int] = UNSET
    _trim_start: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("bases={}".format(repr(self._bases)))
        fields.append("dna_sequence_id={}".format(repr(self._dna_sequence_id)))
        fields.append("name={}".format(repr(self._name)))
        fields.append("pairwise_identity={}".format(repr(self._pairwise_identity)))
        fields.append("trim_end={}".format(repr(self._trim_end)))
        fields.append("trim_start={}".format(repr(self._trim_start)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "AlignedSequence({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        bases = self._bases
        dna_sequence_id = self._dna_sequence_id
        name = self._name
        pairwise_identity = self._pairwise_identity
        trim_end = self._trim_end
        trim_start = self._trim_start

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if bases is not UNSET:
            field_dict["bases"] = bases
        if dna_sequence_id is not UNSET:
            field_dict["dnaSequenceId"] = dna_sequence_id
        if name is not UNSET:
            field_dict["name"] = name
        if pairwise_identity is not UNSET:
            field_dict["pairwiseIdentity"] = pairwise_identity
        if trim_end is not UNSET:
            field_dict["trimEnd"] = trim_end
        if trim_start is not UNSET:
            field_dict["trimStart"] = trim_start

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_bases() -> Union[Unset, str]:
            bases = d.pop("bases")
            return bases

        try:
            bases = get_bases()
        except KeyError:
            if strict:
                raise
            bases = cast(Union[Unset, str], UNSET)

        def get_dna_sequence_id() -> Union[Unset, None, str]:
            dna_sequence_id = d.pop("dnaSequenceId")
            return dna_sequence_id

        try:
            dna_sequence_id = get_dna_sequence_id()
        except KeyError:
            if strict:
                raise
            dna_sequence_id = cast(Union[Unset, None, str], UNSET)

        def get_name() -> Union[Unset, str]:
            name = d.pop("name")
            return name

        try:
            name = get_name()
        except KeyError:
            if strict:
                raise
            name = cast(Union[Unset, str], UNSET)

        def get_pairwise_identity() -> Union[Unset, float]:
            pairwise_identity = d.pop("pairwiseIdentity")
            return pairwise_identity

        try:
            pairwise_identity = get_pairwise_identity()
        except KeyError:
            if strict:
                raise
            pairwise_identity = cast(Union[Unset, float], UNSET)

        def get_trim_end() -> Union[Unset, int]:
            trim_end = d.pop("trimEnd")
            return trim_end

        try:
            trim_end = get_trim_end()
        except KeyError:
            if strict:
                raise
            trim_end = cast(Union[Unset, int], UNSET)

        def get_trim_start() -> Union[Unset, int]:
            trim_start = d.pop("trimStart")
            return trim_start

        try:
            trim_start = get_trim_start()
        except KeyError:
            if strict:
                raise
            trim_start = cast(Union[Unset, int], UNSET)

        aligned_sequence = cls(
            bases=bases,
            dna_sequence_id=dna_sequence_id,
            name=name,
            pairwise_identity=pairwise_identity,
            trim_end=trim_end,
            trim_start=trim_start,
        )

        aligned_sequence.additional_properties = d
        return aligned_sequence

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def bases(self) -> str:
        if isinstance(self._bases, Unset):
            raise NotPresentError(self, "bases")
        return self._bases

    @bases.setter
    def bases(self, value: str) -> None:
        self._bases = value

    @bases.deleter
    def bases(self) -> None:
        self._bases = UNSET

    @property
    def dna_sequence_id(self) -> Optional[str]:
        if isinstance(self._dna_sequence_id, Unset):
            raise NotPresentError(self, "dna_sequence_id")
        return self._dna_sequence_id

    @dna_sequence_id.setter
    def dna_sequence_id(self, value: Optional[str]) -> None:
        self._dna_sequence_id = value

    @dna_sequence_id.deleter
    def dna_sequence_id(self) -> None:
        self._dna_sequence_id = UNSET

    @property
    def name(self) -> str:
        if isinstance(self._name, Unset):
            raise NotPresentError(self, "name")
        return self._name

    @name.setter
    def name(self, value: str) -> None:
        self._name = value

    @name.deleter
    def name(self) -> None:
        self._name = UNSET

    @property
    def pairwise_identity(self) -> float:
        """Fraction of bases between trimStart and trimEnd that match the template bases. Only present for Template Alignments; Will be empty for Consensus Alignments."""
        if isinstance(self._pairwise_identity, Unset):
            raise NotPresentError(self, "pairwise_identity")
        return self._pairwise_identity

    @pairwise_identity.setter
    def pairwise_identity(self, value: float) -> None:
        self._pairwise_identity = value

    @pairwise_identity.deleter
    def pairwise_identity(self) -> None:
        self._pairwise_identity = UNSET

    @property
    def trim_end(self) -> int:
        if isinstance(self._trim_end, Unset):
            raise NotPresentError(self, "trim_end")
        return self._trim_end

    @trim_end.setter
    def trim_end(self, value: int) -> None:
        self._trim_end = value

    @trim_end.deleter
    def trim_end(self) -> None:
        self._trim_end = UNSET

    @property
    def trim_start(self) -> int:
        if isinstance(self._trim_start, Unset):
            raise NotPresentError(self, "trim_start")
        return self._trim_start

    @trim_start.setter
    def trim_start(self, value: int) -> None:
        self._trim_start = value

    @trim_start.deleter
    def trim_start(self) -> None:
        self._trim_start = UNSET



from typing import Any, cast, Dict, List, Type, TypeVar

import attr

from ..extensions import NotPresentError
from ..models.entity_archive_reason import EntityArchiveReason
from ..types import UNSET, Unset

T = TypeVar("T", bound="MixturesArchive")


@attr.s(auto_attribs=True, repr=False)
class MixturesArchive:
    """The request body for archiving mixtures."""

    _mixture_ids: List[str]
    _reason: EntityArchiveReason

    def __repr__(self):
        fields = []
        fields.append("mixture_ids={}".format(repr(self._mixture_ids)))
        fields.append("reason={}".format(repr(self._reason)))
        return "MixturesArchive({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        mixture_ids = self._mixture_ids

        reason = self._reason.value

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if mixture_ids is not UNSET:
            field_dict["mixtureIds"] = mixture_ids
        if reason is not UNSET:
            field_dict["reason"] = reason

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_mixture_ids() -> List[str]:
            mixture_ids = cast(List[str], d.pop("mixtureIds"))

            return mixture_ids

        try:
            mixture_ids = get_mixture_ids()
        except KeyError:
            if strict:
                raise
            mixture_ids = cast(List[str], UNSET)

        def get_reason() -> EntityArchiveReason:
            _reason = d.pop("reason")
            try:
                reason = EntityArchiveReason(_reason)
            except ValueError:
                reason = EntityArchiveReason.of_unknown(_reason)

            return reason

        try:
            reason = get_reason()
        except KeyError:
            if strict:
                raise
            reason = cast(EntityArchiveReason, UNSET)

        mixtures_archive = cls(
            mixture_ids=mixture_ids,
            reason=reason,
        )

        return mixtures_archive

    @property
    def mixture_ids(self) -> List[str]:
        if isinstance(self._mixture_ids, Unset):
            raise NotPresentError(self, "mixture_ids")
        return self._mixture_ids

    @mixture_ids.setter
    def mixture_ids(self, value: List[str]) -> None:
        self._mixture_ids = value

    @property
    def reason(self) -> EntityArchiveReason:
        """The reason for archiving the provided entities. Accepted reasons may differ based on tenant configuration."""
        if isinstance(self._reason, Unset):
            raise NotPresentError(self, "reason")
        return self._reason

    @reason.setter
    def reason(self, value: EntityArchiveReason) -> None:
        self._reason = value



from typing import Any, Dict, Optional, Union

import httpx

from ...client import Client
from ...models.async_task_link import AsyncTaskLink
from ...models.bad_request_error import BadRequestError
from ...models.register_entities import RegisterEntities
from ...types import Response


def _get_kwargs(
    *,
    client: Client,
    registry_id: str,
    json_body: RegisterEntities,
) -> Dict[str, Any]:
    url = "{}/registries/{registry_id}:bulk-register-entities".format(
        client.base_url, registry_id=registry_id
    )

    headers: Dict[str, Any] = client.httpx_client.headers
    headers.update(client.get_headers())

    cookies: Dict[str, Any] = client.httpx_client.cookies
    cookies.update(client.get_cookies())

    json_json_body = json_body.to_dict()

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "json": json_json_body,
    }


def _parse_response(*, response: httpx.Response) -> Optional[Union[AsyncTaskLink, BadRequestError]]:
    if response.status_code == 202:
        response_202 = AsyncTaskLink.from_dict(response.json(), strict=False)

        return response_202
    if response.status_code == 400:
        response_400 = BadRequestError.from_dict(response.json(), strict=False)

        return response_400
    return None


def _build_response(*, response: httpx.Response) -> Response[Union[AsyncTaskLink, BadRequestError]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    registry_id: str,
    json_body: RegisterEntities,
) -> Response[Union[AsyncTaskLink, BadRequestError]]:
    kwargs = _get_kwargs(
        client=client,
        registry_id=registry_id,
        json_body=json_body,
    )

    response = client.httpx_client.post(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    registry_id: str,
    json_body: RegisterEntities,
) -> Optional[Union[AsyncTaskLink, BadRequestError]]:
    """Attempts to move entities into the registry. Limit of 2500 entities per request. This endpoint will first check that the entities are all valid to be moved into the registry, given the namingStrategy. If any entities fail validation, no files will be moved and errors describing invalid entities is returned. If all entities pass validation, the entities are moved into the registry."""

    return sync_detailed(
        client=client,
        registry_id=registry_id,
        json_body=json_body,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    registry_id: str,
    json_body: RegisterEntities,
) -> Response[Union[AsyncTaskLink, BadRequestError]]:
    kwargs = _get_kwargs(
        client=client,
        registry_id=registry_id,
        json_body=json_body,
    )

    async with httpx.AsyncClient() as _client:
        response = await _client.post(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    registry_id: str,
    json_body: RegisterEntities,
) -> Optional[Union[AsyncTaskLink, BadRequestError]]:
    """Attempts to move entities into the registry. Limit of 2500 entities per request. This endpoint will first check that the entities are all valid to be moved into the registry, given the namingStrategy. If any entities fail validation, no files will be moved and errors describing invalid entities is returned. If all entities pass validation, the entities are moved into the registry."""

    return (
        await asyncio_detailed(
            client=client,
            registry_id=registry_id,
            json_body=json_body,
        )
    ).parsed



from typing import Union

from ..extensions import UnknownType
from ..models.lifecycle_activate_webhook import LifecycleActivateWebhook
from ..models.lifecycle_deactivate_webhook import LifecycleDeactivateWebhook

WebhookMessage = Union[LifecycleActivateWebhook, LifecycleDeactivateWebhook, UnknownType]



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.blob import Blob
from ..types import UNSET, Unset

T = TypeVar("T", bound="BlobsBulkGet")


@attr.s(auto_attribs=True, repr=False)
class BlobsBulkGet:
    """  """

    _blobs: Union[Unset, List[Blob]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("blobs={}".format(repr(self._blobs)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "BlobsBulkGet({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        blobs: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._blobs, Unset):
            blobs = []
            for blobs_item_data in self._blobs:
                blobs_item = blobs_item_data.to_dict()

                blobs.append(blobs_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if blobs is not UNSET:
            field_dict["blobs"] = blobs

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_blobs() -> Union[Unset, List[Blob]]:
            blobs = []
            _blobs = d.pop("blobs")
            for blobs_item_data in _blobs or []:
                blobs_item = Blob.from_dict(blobs_item_data, strict=False)

                blobs.append(blobs_item)

            return blobs

        try:
            blobs = get_blobs()
        except KeyError:
            if strict:
                raise
            blobs = cast(Union[Unset, List[Blob]], UNSET)

        blobs_bulk_get = cls(
            blobs=blobs,
        )

        blobs_bulk_get.additional_properties = d
        return blobs_bulk_get

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def blobs(self) -> List[Blob]:
        if isinstance(self._blobs, Unset):
            raise NotPresentError(self, "blobs")
        return self._blobs

    @blobs.setter
    def blobs(self, value: List[Blob]) -> None:
        self._blobs = value

    @blobs.deleter
    def blobs(self) -> None:
        self._blobs = UNSET



from typing import Union

from ..extensions import UnknownType
from ..models.canvas_initialize_webhook import CanvasInitializeWebhook
from ..models.canvas_interaction_webhook import CanvasInteractionWebhook

WebhookMessage = Union[CanvasInteractionWebhook, CanvasInitializeWebhook, UnknownType]



from typing import Any, cast, Dict, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.text_label_ui_block_type import TextLabelUiBlockType
from ..types import UNSET, Unset

T = TypeVar("T", bound="TextLabelUiBlock")


@attr.s(auto_attribs=True, repr=False)
class TextLabelUiBlock:
    """ Deprecated, slated for removal March 3, 2023. Use MarkdownUiBlock instead. """

    _text: str
    _type: TextLabelUiBlockType
    _id: Union[Unset, str] = UNSET

    def __repr__(self):
        fields = []
        fields.append("text={}".format(repr(self._text)))
        fields.append("type={}".format(repr(self._type)))
        fields.append("id={}".format(repr(self._id)))
        return "TextLabelUiBlock({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        text = self._text
        type = self._type.value

        id = self._id

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if text is not UNSET:
            field_dict["text"] = text
        if type is not UNSET:
            field_dict["type"] = type
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_text() -> str:
            text = d.pop("text")
            return text

        try:
            text = get_text()
        except KeyError:
            if strict:
                raise
            text = cast(str, UNSET)

        def get_type() -> TextLabelUiBlockType:
            _type = d.pop("type")
            try:
                type = TextLabelUiBlockType(_type)
            except ValueError:
                type = TextLabelUiBlockType.of_unknown(_type)

            return type

        try:
            type = get_type()
        except KeyError:
            if strict:
                raise
            type = cast(TextLabelUiBlockType, UNSET)

        def get_id() -> Union[Unset, str]:
            id = d.pop("id")
            return id

        try:
            id = get_id()
        except KeyError:
            if strict:
                raise
            id = cast(Union[Unset, str], UNSET)

        text_label_ui_block = cls(
            text=text,
            type=type,
            id=id,
        )

        return text_label_ui_block

    @property
    def text(self) -> str:
        if isinstance(self._text, Unset):
            raise NotPresentError(self, "text")
        return self._text

    @text.setter
    def text(self, value: str) -> None:
        self._text = value

    @property
    def type(self) -> TextLabelUiBlockType:
        if isinstance(self._type, Unset):
            raise NotPresentError(self, "type")
        return self._type

    @type.setter
    def type(self, value: TextLabelUiBlockType) -> None:
        self._type = value

    @property
    def id(self) -> str:
        if isinstance(self._id, Unset):
            raise NotPresentError(self, "id")
        return self._id

    @id.setter
    def id(self, value: str) -> None:
        self._id = value

    @id.deleter
    def id(self) -> None:
        self._id = UNSET



from enum import Enum
from functools import lru_cache
from typing import cast

from ..extensions import Enums


class SessionMessageStyle(Enums.KnownString):
    ERROR = "ERROR"
    INFO = "INFO"
    NONE = "NONE"
    SUCCESS = "SUCCESS"
    WARNING = "WARNING"

    def __str__(self) -> str:
        return str(self.value)

    @staticmethod
    @lru_cache(maxsize=None)
    def of_unknown(val: str) -> "SessionMessageStyle":
        if not isinstance(val, str):
            raise ValueError(f"Value of SessionMessageStyle must be a string (encountered: {val})")
        newcls = Enum("SessionMessageStyle", {"_UNKNOWN": val}, type=Enums.UnknownString)  # type: ignore
        return cast(SessionMessageStyle, getattr(newcls, "_UNKNOWN"))



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.token_response_token_type import TokenResponseTokenType
from ..types import UNSET, Unset

T = TypeVar("T", bound="TokenResponse")


@attr.s(auto_attribs=True, repr=False)
class TokenResponse:
    """  """

    _access_token: Union[Unset, str] = UNSET
    _expires_in: Union[Unset, int] = UNSET
    _token_type: Union[Unset, TokenResponseTokenType] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("access_token={}".format(repr(self._access_token)))
        fields.append("expires_in={}".format(repr(self._expires_in)))
        fields.append("token_type={}".format(repr(self._token_type)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "TokenResponse({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        access_token = self._access_token
        expires_in = self._expires_in
        token_type: Union[Unset, int] = UNSET
        if not isinstance(self._token_type, Unset):
            token_type = self._token_type.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if access_token is not UNSET:
            field_dict["access_token"] = access_token
        if expires_in is not UNSET:
            field_dict["expires_in"] = expires_in
        if token_type is not UNSET:
            field_dict["token_type"] = token_type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_access_token() -> Union[Unset, str]:
            access_token = d.pop("access_token")
            return access_token

        try:
            access_token = get_access_token()
        except KeyError:
            if strict:
                raise
            access_token = cast(Union[Unset, str], UNSET)

        def get_expires_in() -> Union[Unset, int]:
            expires_in = d.pop("expires_in")
            return expires_in

        try:
            expires_in = get_expires_in()
        except KeyError:
            if strict:
                raise
            expires_in = cast(Union[Unset, int], UNSET)

        def get_token_type() -> Union[Unset, TokenResponseTokenType]:
            token_type = UNSET
            _token_type = d.pop("token_type")
            if _token_type is not None and _token_type is not UNSET:
                try:
                    token_type = TokenResponseTokenType(_token_type)
                except ValueError:
                    token_type = TokenResponseTokenType.of_unknown(_token_type)

            return token_type

        try:
            token_type = get_token_type()
        except KeyError:
            if strict:
                raise
            token_type = cast(Union[Unset, TokenResponseTokenType], UNSET)

        token_response = cls(
            access_token=access_token,
            expires_in=expires_in,
            token_type=token_type,
        )

        token_response.additional_properties = d
        return token_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def access_token(self) -> str:
        if isinstance(self._access_token, Unset):
            raise NotPresentError(self, "access_token")
        return self._access_token

    @access_token.setter
    def access_token(self, value: str) -> None:
        self._access_token = value

    @access_token.deleter
    def access_token(self) -> None:
        self._access_token = UNSET

    @property
    def expires_in(self) -> int:
        """ Number of seconds that token is valid for """
        if isinstance(self._expires_in, Unset):
            raise NotPresentError(self, "expires_in")
        return self._expires_in

    @expires_in.setter
    def expires_in(self, value: int) -> None:
        self._expires_in = value

    @expires_in.deleter
    def expires_in(self) -> None:
        self._expires_in = UNSET

    @property
    def token_type(self) -> TokenResponseTokenType:
        if isinstance(self._token_type, Unset):
            raise NotPresentError(self, "token_type")
        return self._token_type

    @token_type.setter
    def token_type(self, value: TokenResponseTokenType) -> None:
        self._token_type = value

    @token_type.deleter
    def token_type(self) -> None:
        self._token_type = UNSET



import datetime
from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..extensions import NotPresentError
from ..models.sample_group import SampleGroup
from ..models.sample_group_status import SampleGroupStatus
from ..types import UNSET, Unset

T = TypeVar("T", bound="RequestFulfillment")


@attr.s(auto_attribs=True, repr=False)
class RequestFulfillment:
    """A request fulfillment represents work that is done which changes the status of a request or a sample group within that request.
    Fulfillments are created when state changes between IN_PROGRESS, COMPLETED, or FAILED statuses. Fulfillments do not capture a PENDING state because all requests or request sample groups are considered PENDING until the first corresponding fulfillment is created.
    """

    _created_at: Union[Unset, datetime.datetime] = UNSET
    _entry_id: Union[Unset, str] = UNSET
    _id: Union[Unset, str] = UNSET
    _modified_at: Union[Unset, datetime.datetime] = UNSET
    _request_id: Union[Unset, str] = UNSET
    _sample_group: Union[Unset, None, SampleGroup] = UNSET
    _status: Union[Unset, SampleGroupStatus] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("created_at={}".format(repr(self._created_at)))
        fields.append("entry_id={}".format(repr(self._entry_id)))
        fields.append("id={}".format(repr(self._id)))
        fields.append("modified_at={}".format(repr(self._modified_at)))
        fields.append("request_id={}".format(repr(self._request_id)))
        fields.append("sample_group={}".format(repr(self._sample_group)))
        fields.append("status={}".format(repr(self._status)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "RequestFulfillment({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        created_at: Union[Unset, str] = UNSET
        if not isinstance(self._created_at, Unset):
            created_at = self._created_at.isoformat()

        entry_id = self._entry_id
        id = self._id
        modified_at: Union[Unset, str] = UNSET
        if not isinstance(self._modified_at, Unset):
            modified_at = self._modified_at.isoformat()

        request_id = self._request_id
        sample_group: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self._sample_group, Unset):
            sample_group = self._sample_group.to_dict() if self._sample_group else None

        status: Union[Unset, int] = UNSET
        if not isinstance(self._status, Unset):
            status = self._status.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if created_at is not UNSET:
            field_dict["createdAt"] = created_at
        if entry_id is not UNSET:
            field_dict["entryId"] = entry_id
        if id is not UNSET:
            field_dict["id"] = id
        if modified_at is not UNSET:
            field_dict["modifiedAt"] = modified_at
        if request_id is not UNSET:
            field_dict["requestId"] = request_id
        if sample_group is not UNSET:
            field_dict["sampleGroup"] = sample_group
        if status is not UNSET:
            field_dict["status"] = status

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_created_at() -> Union[Unset, datetime.datetime]:
            created_at: Union[Unset, datetime.datetime] = UNSET
            _created_at = d.pop("createdAt")
            if _created_at is not None and not isinstance(_created_at, Unset):
                created_at = isoparse(cast(str, _created_at))

            return created_at

        try:
            created_at = get_created_at()
        except KeyError:
            if strict:
                raise
            created_at = cast(Union[Unset, datetime.datetime], UNSET)

        def get_entry_id() -> Union[Unset, str]:
            entry_id = d.pop("entryId")
            return entry_id

        try:
            entry_id = get_entry_id()
        except KeyError:
            if strict:
                raise
            entry_id = cast(Union[Unset, str], UNSET)

        def get_id() -> Union[Unset, str]:
            id = d.pop("id")
            return id

        try:
            id = get_id()
        except KeyError:
            if strict:
                raise
            id = cast(Union[Unset, str], UNSET)

        def get_modified_at() -> Union[Unset, datetime.datetime]:
            modified_at: Union[Unset, datetime.datetime] = UNSET
            _modified_at = d.pop("modifiedAt")
            if _modified_at is not None and not isinstance(_modified_at, Unset):
                modified_at = isoparse(cast(str, _modified_at))

            return modified_at

        try:
            modified_at = get_modified_at()
        except KeyError:
            if strict:
                raise
            modified_at = cast(Union[Unset, datetime.datetime], UNSET)

        def get_request_id() -> Union[Unset, str]:
            request_id = d.pop("requestId")
            return request_id

        try:
            request_id = get_request_id()
        except KeyError:
            if strict:
                raise
            request_id = cast(Union[Unset, str], UNSET)

        def get_sample_group() -> Union[Unset, None, SampleGroup]:
            sample_group = None
            _sample_group = d.pop("sampleGroup")

            if _sample_group is not None and not isinstance(_sample_group, Unset):
                sample_group = SampleGroup.from_dict(_sample_group)

            return sample_group

        try:
            sample_group = get_sample_group()
        except KeyError:
            if strict:
                raise
            sample_group = cast(Union[Unset, None, SampleGroup], UNSET)

        def get_status() -> Union[Unset, SampleGroupStatus]:
            status = UNSET
            _status = d.pop("status")
            if _status is not None and _status is not UNSET:
                try:
                    status = SampleGroupStatus(_status)
                except ValueError:
                    status = SampleGroupStatus.of_unknown(_status)

            return status

        try:
            status = get_status()
        except KeyError:
            if strict:
                raise
            status = cast(Union[Unset, SampleGroupStatus], UNSET)

        request_fulfillment = cls(
            created_at=created_at,
            entry_id=entry_id,
            id=id,
            modified_at=modified_at,
            request_id=request_id,
            sample_group=sample_group,
            status=status,
        )

        request_fulfillment.additional_properties = d
        return request_fulfillment

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def created_at(self) -> datetime.datetime:
        """ Date and time the fulfillment was created """
        if isinstance(self._created_at, Unset):
            raise NotPresentError(self, "created_at")
        return self._created_at

    @created_at.setter
    def created_at(self, value: datetime.datetime) -> None:
        self._created_at = value

    @created_at.deleter
    def created_at(self) -> None:
        self._created_at = UNSET

    @property
    def entry_id(self) -> str:
        """ ID of the entry this fulfillment was executed in, if any """
        if isinstance(self._entry_id, Unset):
            raise NotPresentError(self, "entry_id")
        return self._entry_id

    @entry_id.setter
    def entry_id(self, value: str) -> None:
        self._entry_id = value

    @entry_id.deleter
    def entry_id(self) -> None:
        self._entry_id = UNSET

    @property
    def id(self) -> str:
        """ ID of the request fulfillment """
        if isinstance(self._id, Unset):
            raise NotPresentError(self, "id")
        return self._id

    @id.setter
    def id(self, value: str) -> None:
        self._id = value

    @id.deleter
    def id(self) -> None:
        self._id = UNSET

    @property
    def modified_at(self) -> datetime.datetime:
        """ DateTime the Request Fulfillment was last modified """
        if isinstance(self._modified_at, Unset):
            raise NotPresentError(self, "modified_at")
        return self._modified_at

    @modified_at.setter
    def modified_at(self, value: datetime.datetime) -> None:
        self._modified_at = value

    @modified_at.deleter
    def modified_at(self) -> None:
        self._modified_at = UNSET

    @property
    def request_id(self) -> str:
        """ ID of the request this fulfillment fulfills """
        if isinstance(self._request_id, Unset):
            raise NotPresentError(self, "request_id")
        return self._request_id

    @request_id.setter
    def request_id(self, value: str) -> None:
        self._request_id = value

    @request_id.deleter
    def request_id(self) -> None:
        self._request_id = UNSET

    @property
    def sample_group(self) -> Optional[SampleGroup]:
        """ Represents a sample group that is an input to a request. A sample group is a set of samples upon which work in the request should be done. """
        if isinstance(self._sample_group, Unset):
            raise NotPresentError(self, "sample_group")
        return self._sample_group

    @sample_group.setter
    def sample_group(self, value: Optional[SampleGroup]) -> None:
        self._sample_group = value

    @sample_group.deleter
    def sample_group(self) -> None:
        self._sample_group = UNSET

    @property
    def status(self) -> SampleGroupStatus:
        """ Status of a sample group """
        if isinstance(self._status, Unset):
            raise NotPresentError(self, "status")
        return self._status

    @status.setter
    def status(self, value: SampleGroupStatus) -> None:
        self._status = value

    @status.deleter
    def status(self) -> None:
        self._status = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.entry_table_row import EntryTableRow
from ..types import UNSET, Unset

T = TypeVar("T", bound="EntryTable")


@attr.s(auto_attribs=True, repr=False)
class EntryTable:
    """Actual tabular data with rows and columns of text on the note."""

    _column_labels: Union[Unset, List[Optional[str]]] = UNSET
    _name: Union[Unset, str] = UNSET
    _rows: Union[Unset, List[EntryTableRow]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("column_labels={}".format(repr(self._column_labels)))
        fields.append("name={}".format(repr(self._name)))
        fields.append("rows={}".format(repr(self._rows)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "EntryTable({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        column_labels: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._column_labels, Unset):
            column_labels = self._column_labels

        name = self._name
        rows: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._rows, Unset):
            rows = []
            for rows_item_data in self._rows:
                rows_item = rows_item_data.to_dict()

                rows.append(rows_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if column_labels is not UNSET:
            field_dict["columnLabels"] = column_labels
        if name is not UNSET:
            field_dict["name"] = name
        if rows is not UNSET:
            field_dict["rows"] = rows

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_column_labels() -> Union[Unset, List[Optional[str]]]:
            column_labels = cast(List[Optional[str]], d.pop("columnLabels"))

            return column_labels

        try:
            column_labels = get_column_labels()
        except KeyError:
            if strict:
                raise
            column_labels = cast(Union[Unset, List[Optional[str]]], UNSET)

        def get_name() -> Union[Unset, str]:
            name = d.pop("name")
            return name

        try:
            name = get_name()
        except KeyError:
            if strict:
                raise
            name = cast(Union[Unset, str], UNSET)

        def get_rows() -> Union[Unset, List[EntryTableRow]]:
            rows = []
            _rows = d.pop("rows")
            for rows_item_data in _rows or []:
                rows_item = EntryTableRow.from_dict(rows_item_data, strict=False)

                rows.append(rows_item)

            return rows

        try:
            rows = get_rows()
        except KeyError:
            if strict:
                raise
            rows = cast(Union[Unset, List[EntryTableRow]], UNSET)

        entry_table = cls(
            column_labels=column_labels,
            name=name,
            rows=rows,
        )

        entry_table.additional_properties = d
        return entry_table

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def column_labels(self) -> List[Optional[str]]:
        """Array of strings, with one item per column. Defaults to null, if the user is using the default, but is set if the user has given a custom name to the column."""
        if isinstance(self._column_labels, Unset):
            raise NotPresentError(self, "column_labels")
        return self._column_labels

    @column_labels.setter
    def column_labels(self, value: List[Optional[str]]) -> None:
        self._column_labels = value

    @column_labels.deleter
    def column_labels(self) -> None:
        self._column_labels = UNSET

    @property
    def name(self) -> str:
        """Name of the table - defaults to e.g. Table1 but can be renamed."""
        if isinstance(self._name, Unset):
            raise NotPresentError(self, "name")
        return self._name

    @name.setter
    def name(self, value: str) -> None:
        self._name = value

    @name.deleter
    def name(self) -> None:
        self._name = UNSET

    @property
    def rows(self) -> List[EntryTableRow]:
        """ Array of row objects. """
        if isinstance(self._rows, Unset):
            raise NotPresentError(self, "rows")
        return self._rows

    @rows.setter
    def rows(self, value: List[EntryTableRow]) -> None:
        self._rows = value

    @rows.deleter
    def rows(self) -> None:
        self._rows = UNSET



from typing import Any, cast, Dict, List, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.collaboration import Collaboration
from ..types import UNSET, Unset

T = TypeVar("T", bound="CollaborationsPaginatedList")


@attr.s(auto_attribs=True, repr=False)
class CollaborationsPaginatedList:
    """  """

    _collaborations: Union[Unset, List[Collaboration]] = UNSET
    _next_token: Union[Unset, str] = UNSET

    def __repr__(self):
        fields = []
        fields.append("collaborations={}".format(repr(self._collaborations)))
        fields.append("next_token={}".format(repr(self._next_token)))
        return "CollaborationsPaginatedList({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        collaborations: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._collaborations, Unset):
            collaborations = []
            for collaborations_item_data in self._collaborations:
                collaborations_item = collaborations_item_data.to_dict()

                collaborations.append(collaborations_item)

        next_token = self._next_token

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if collaborations is not UNSET:
            field_dict["collaborations"] = collaborations
        if next_token is not UNSET:
            field_dict["nextToken"] = next_token

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_collaborations() -> Union[Unset, List[Collaboration]]:
            collaborations = []
            _collaborations = d.pop("collaborations")
            for collaborations_item_data in _collaborations or []:
                collaborations_item = Collaboration.from_dict(collaborations_item_data, strict=False)

                collaborations.append(collaborations_item)

            return collaborations

        try:
            collaborations = get_collaborations()
        except KeyError:
            if strict:
                raise
            collaborations = cast(Union[Unset, List[Collaboration]], UNSET)

        def get_next_token() -> Union[Unset, str]:
            next_token = d.pop("nextToken")
            return next_token

        try:
            next_token = get_next_token()
        except KeyError:
            if strict:
                raise
            next_token = cast(Union[Unset, str], UNSET)

        collaborations_paginated_list = cls(
            collaborations=collaborations,
            next_token=next_token,
        )

        return collaborations_paginated_list

    @property
    def collaborations(self) -> List[Collaboration]:
        if isinstance(self._collaborations, Unset):
            raise NotPresentError(self, "collaborations")
        return self._collaborations

    @collaborations.setter
    def collaborations(self, value: List[Collaboration]) -> None:
        self._collaborations = value

    @collaborations.deleter
    def collaborations(self) -> None:
        self._collaborations = UNSET

    @property
    def next_token(self) -> str:
        if isinstance(self._next_token, Unset):
            raise NotPresentError(self, "next_token")
        return self._next_token

    @next_token.setter
    def next_token(self, value: str) -> None:
        self._next_token = value

    @next_token.deleter
    def next_token(self) -> None:
        self._next_token = UNSET



from typing import Any, cast, Dict, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.default_concentration_summary import DefaultConcentrationSummary
from ..models.fields import Fields
from ..types import UNSET, Unset

T = TypeVar("T", bound="BatchCreate")


@attr.s(auto_attribs=True, repr=False)
class BatchCreate:
    """  """

    _default_concentration: Union[Unset, DefaultConcentrationSummary] = UNSET
    _entity_id: Union[Unset, str] = UNSET
    _fields: Union[Unset, Fields] = UNSET

    def __repr__(self):
        fields = []
        fields.append("default_concentration={}".format(repr(self._default_concentration)))
        fields.append("entity_id={}".format(repr(self._entity_id)))
        fields.append("fields={}".format(repr(self._fields)))
        return "BatchCreate({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        default_concentration: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._default_concentration, Unset):
            default_concentration = self._default_concentration.to_dict()

        entity_id = self._entity_id
        fields: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._fields, Unset):
            fields = self._fields.to_dict()

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if default_concentration is not UNSET:
            field_dict["defaultConcentration"] = default_concentration
        if entity_id is not UNSET:
            field_dict["entityId"] = entity_id
        if fields is not UNSET:
            field_dict["fields"] = fields

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_default_concentration() -> Union[Unset, DefaultConcentrationSummary]:
            default_concentration: Union[Unset, Union[Unset, DefaultConcentrationSummary]] = UNSET
            _default_concentration = d.pop("defaultConcentration")

            if not isinstance(_default_concentration, Unset):
                default_concentration = DefaultConcentrationSummary.from_dict(_default_concentration)

            return default_concentration

        try:
            default_concentration = get_default_concentration()
        except KeyError:
            if strict:
                raise
            default_concentration = cast(Union[Unset, DefaultConcentrationSummary], UNSET)

        def get_entity_id() -> Union[Unset, str]:
            entity_id = d.pop("entityId")
            return entity_id

        try:
            entity_id = get_entity_id()
        except KeyError:
            if strict:
                raise
            entity_id = cast(Union[Unset, str], UNSET)

        def get_fields() -> Union[Unset, Fields]:
            fields: Union[Unset, Union[Unset, Fields]] = UNSET
            _fields = d.pop("fields")

            if not isinstance(_fields, Unset):
                fields = Fields.from_dict(_fields)

            return fields

        try:
            fields = get_fields()
        except KeyError:
            if strict:
                raise
            fields = cast(Union[Unset, Fields], UNSET)

        batch_create = cls(
            default_concentration=default_concentration,
            entity_id=entity_id,
            fields=fields,
        )

        return batch_create

    @property
    def default_concentration(self) -> DefaultConcentrationSummary:
        if isinstance(self._default_concentration, Unset):
            raise NotPresentError(self, "default_concentration")
        return self._default_concentration

    @default_concentration.setter
    def default_concentration(self, value: DefaultConcentrationSummary) -> None:
        self._default_concentration = value

    @default_concentration.deleter
    def default_concentration(self) -> None:
        self._default_concentration = UNSET

    @property
    def entity_id(self) -> str:
        """ API identifier for the entity that the batch will be added to. """
        if isinstance(self._entity_id, Unset):
            raise NotPresentError(self, "entity_id")
        return self._entity_id

    @entity_id.setter
    def entity_id(self, value: str) -> None:
        self._entity_id = value

    @entity_id.deleter
    def entity_id(self) -> None:
        self._entity_id = UNSET

    @property
    def fields(self) -> Fields:
        if isinstance(self._fields, Unset):
            raise NotPresentError(self, "fields")
        return self._fields

    @fields.setter
    def fields(self, value: Fields) -> None:
        self._fields = value

    @fields.deleter
    def fields(self) -> None:
        self._fields = UNSET



import warnings

warnings.warn(
    "This package is deprecated: please use benchling_sdk.models if using the SDK, "
    "or benchling_api_client.v2.stable.models otherwise.",
    DeprecationWarning,
)



from typing import Any, Dict, Type, TypeVar

import attr

T = TypeVar("T", bound="ArchiveRecordSet")


@attr.s(auto_attribs=True, repr=False)
class ArchiveRecordSet:
    """ Currently, we only support setting a null value for archiveRecord, which unarchives the item """

    def __repr__(self):
        fields = []
        return "ArchiveRecordSet({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:

        field_dict: Dict[str, Any] = {}

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        src_dict.copy()

        archive_record_set = cls()

        return archive_record_set



import datetime
from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..extensions import NotPresentError
from ..models.fields import Fields
from ..types import UNSET, Unset

T = TypeVar("T", bound="WorkflowTaskWriteBase")


@attr.s(auto_attribs=True, repr=False)
class WorkflowTaskWriteBase:
    """  """

    _assignee_id: Union[Unset, str] = UNSET
    _fields: Union[Unset, Fields] = UNSET
    _scheduled_on: Union[Unset, datetime.date] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("assignee_id={}".format(repr(self._assignee_id)))
        fields.append("fields={}".format(repr(self._fields)))
        fields.append("scheduled_on={}".format(repr(self._scheduled_on)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "WorkflowTaskWriteBase({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        assignee_id = self._assignee_id
        fields: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._fields, Unset):
            fields = self._fields.to_dict()

        scheduled_on: Union[Unset, str] = UNSET
        if not isinstance(self._scheduled_on, Unset):
            scheduled_on = self._scheduled_on.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if assignee_id is not UNSET:
            field_dict["assigneeId"] = assignee_id
        if fields is not UNSET:
            field_dict["fields"] = fields
        if scheduled_on is not UNSET:
            field_dict["scheduledOn"] = scheduled_on

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_assignee_id() -> Union[Unset, str]:
            assignee_id = d.pop("assigneeId")
            return assignee_id

        try:
            assignee_id = get_assignee_id()
        except KeyError:
            if strict:
                raise
            assignee_id = cast(Union[Unset, str], UNSET)

        def get_fields() -> Union[Unset, Fields]:
            fields: Union[Unset, Union[Unset, Fields]] = UNSET
            _fields = d.pop("fields")

            if not isinstance(_fields, Unset):
                fields = Fields.from_dict(_fields)

            return fields

        try:
            fields = get_fields()
        except KeyError:
            if strict:
                raise
            fields = cast(Union[Unset, Fields], UNSET)

        def get_scheduled_on() -> Union[Unset, datetime.date]:
            scheduled_on: Union[Unset, datetime.date] = UNSET
            _scheduled_on = d.pop("scheduledOn")
            if _scheduled_on is not None and not isinstance(_scheduled_on, Unset):
                scheduled_on = isoparse(cast(str, _scheduled_on)).date()

            return scheduled_on

        try:
            scheduled_on = get_scheduled_on()
        except KeyError:
            if strict:
                raise
            scheduled_on = cast(Union[Unset, datetime.date], UNSET)

        workflow_task_write_base = cls(
            assignee_id=assignee_id,
            fields=fields,
            scheduled_on=scheduled_on,
        )

        workflow_task_write_base.additional_properties = d
        return workflow_task_write_base

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def assignee_id(self) -> str:
        """ The id of the user assigned to the task """
        if isinstance(self._assignee_id, Unset):
            raise NotPresentError(self, "assignee_id")
        return self._assignee_id

    @assignee_id.setter
    def assignee_id(self, value: str) -> None:
        self._assignee_id = value

    @assignee_id.deleter
    def assignee_id(self) -> None:
        self._assignee_id = UNSET

    @property
    def fields(self) -> Fields:
        if isinstance(self._fields, Unset):
            raise NotPresentError(self, "fields")
        return self._fields

    @fields.setter
    def fields(self, value: Fields) -> None:
        self._fields = value

    @fields.deleter
    def fields(self) -> None:
        self._fields = UNSET

    @property
    def scheduled_on(self) -> datetime.date:
        """ The date on which the task is scheduled to be executed """
        if isinstance(self._scheduled_on, Unset):
            raise NotPresentError(self, "scheduled_on")
        return self._scheduled_on

    @scheduled_on.setter
    def scheduled_on(self, value: datetime.date) -> None:
        self._scheduled_on = value

    @scheduled_on.deleter
    def scheduled_on(self) -> None:
        self._scheduled_on = UNSET



from typing import Any, cast, Dict, List, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.oligo_create_helm import OligoCreateHelm
from ..types import UNSET, Unset

T = TypeVar("T", bound="RnaOligosBulkCreateRequestHelm")


@attr.s(auto_attribs=True, repr=False)
class RnaOligosBulkCreateRequestHelm:
    """  """

    _rna_oligos: Union[Unset, List[OligoCreateHelm]] = UNSET

    def __repr__(self):
        fields = []
        fields.append("rna_oligos={}".format(repr(self._rna_oligos)))
        return "RnaOligosBulkCreateRequestHelm({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        rna_oligos: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._rna_oligos, Unset):
            rna_oligos = []
            for rna_oligos_item_data in self._rna_oligos:
                rna_oligos_item = rna_oligos_item_data.to_dict()

                rna_oligos.append(rna_oligos_item)

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if rna_oligos is not UNSET:
            field_dict["rnaOligos"] = rna_oligos

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_rna_oligos() -> Union[Unset, List[OligoCreateHelm]]:
            rna_oligos = []
            _rna_oligos = d.pop("rnaOligos")
            for rna_oligos_item_data in _rna_oligos or []:
                rna_oligos_item = OligoCreateHelm.from_dict(rna_oligos_item_data, strict=False)

                rna_oligos.append(rna_oligos_item)

            return rna_oligos

        try:
            rna_oligos = get_rna_oligos()
        except KeyError:
            if strict:
                raise
            rna_oligos = cast(Union[Unset, List[OligoCreateHelm]], UNSET)

        rna_oligos_bulk_create_request_helm = cls(
            rna_oligos=rna_oligos,
        )

        return rna_oligos_bulk_create_request_helm

    @property
    def rna_oligos(self) -> List[OligoCreateHelm]:
        if isinstance(self._rna_oligos, Unset):
            raise NotPresentError(self, "rna_oligos")
        return self._rna_oligos

    @rna_oligos.setter
    def rna_oligos(self, value: List[OligoCreateHelm]) -> None:
        self._rna_oligos = value

    @rna_oligos.deleter
    def rna_oligos(self) -> None:
        self._rna_oligos = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError, UnknownType
from ..models.inaccessible_resource import InaccessibleResource
from ..models.linked_app_config_resource_summary import LinkedAppConfigResourceSummary
from ..types import UNSET, Unset

T = TypeVar("T", bound="LinkedAppConfigResourceMixin")


@attr.s(auto_attribs=True, repr=False)
class LinkedAppConfigResourceMixin:
    """  """

    _linked_resource: Union[
        Unset, None, LinkedAppConfigResourceSummary, InaccessibleResource, UnknownType
    ] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("linked_resource={}".format(repr(self._linked_resource)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "LinkedAppConfigResourceMixin({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        linked_resource: Union[Unset, None, Dict[str, Any]]
        if isinstance(self._linked_resource, Unset):
            linked_resource = UNSET
        elif self._linked_resource is None:
            linked_resource = None
        elif isinstance(self._linked_resource, UnknownType):
            linked_resource = self._linked_resource.value
        elif isinstance(self._linked_resource, LinkedAppConfigResourceSummary):
            linked_resource = self._linked_resource.to_dict()

        else:
            linked_resource = self._linked_resource.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if linked_resource is not UNSET:
            field_dict["linkedResource"] = linked_resource

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_linked_resource() -> Union[
            Unset, None, LinkedAppConfigResourceSummary, InaccessibleResource, UnknownType
        ]:
            def _parse_linked_resource(
                data: Union[Unset, None, Dict[str, Any]]
            ) -> Union[Unset, None, LinkedAppConfigResourceSummary, InaccessibleResource, UnknownType]:
                linked_resource: Union[
                    Unset, None, LinkedAppConfigResourceSummary, InaccessibleResource, UnknownType
                ]
                if data is None:
                    return data
                if isinstance(data, Unset):
                    return data
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    linked_app_config_resource = LinkedAppConfigResourceSummary.from_dict(data, strict=True)

                    return linked_app_config_resource
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    linked_app_config_resource = InaccessibleResource.from_dict(data, strict=True)

                    return linked_app_config_resource
                except:  # noqa: E722
                    pass
                return UnknownType(data)

            linked_resource = _parse_linked_resource(d.pop("linkedResource"))

            return linked_resource

        try:
            linked_resource = get_linked_resource()
        except KeyError:
            if strict:
                raise
            linked_resource = cast(
                Union[Unset, None, LinkedAppConfigResourceSummary, InaccessibleResource, UnknownType], UNSET
            )

        linked_app_config_resource_mixin = cls(
            linked_resource=linked_resource,
        )

        linked_app_config_resource_mixin.additional_properties = d
        return linked_app_config_resource_mixin

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def linked_resource(
        self,
    ) -> Optional[Union[LinkedAppConfigResourceSummary, InaccessibleResource, UnknownType]]:
        if isinstance(self._linked_resource, Unset):
            raise NotPresentError(self, "linked_resource")
        return self._linked_resource

    @linked_resource.setter
    def linked_resource(
        self, value: Optional[Union[LinkedAppConfigResourceSummary, InaccessibleResource, UnknownType]]
    ) -> None:
        self._linked_resource = value

    @linked_resource.deleter
    def linked_resource(self) -> None:
        self._linked_resource = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError, UnknownType
from ..models.button_ui_block import ButtonUiBlock
from ..models.chip_ui_block import ChipUiBlock
from ..models.dropdown_ui_block import DropdownUiBlock
from ..models.markdown_ui_block import MarkdownUiBlock
from ..models.search_input_ui_block import SearchInputUiBlock
from ..models.selector_input_ui_block import SelectorInputUiBlock
from ..models.text_input_ui_block import TextInputUiBlock
from ..models.text_label_ui_block import TextLabelUiBlock
from ..types import UNSET, Unset

T = TypeVar("T", bound="CanvasLeafNodeUiBlockList")


@attr.s(auto_attribs=True, repr=False)
class CanvasLeafNodeUiBlockList:
    """  """

    _children: List[
        Union[
            ButtonUiBlock,
            ChipUiBlock,
            DropdownUiBlock,
            MarkdownUiBlock,
            SearchInputUiBlock,
            SelectorInputUiBlock,
            TextInputUiBlock,
            TextLabelUiBlock,
            UnknownType,
        ]
    ]
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("children={}".format(repr(self._children)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "CanvasLeafNodeUiBlockList({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        children = []
        for children_item_data in self._children:
            if isinstance(children_item_data, UnknownType):
                children_item = children_item_data.value
            elif isinstance(children_item_data, ButtonUiBlock):
                children_item = children_item_data.to_dict()

            elif isinstance(children_item_data, ChipUiBlock):
                children_item = children_item_data.to_dict()

            elif isinstance(children_item_data, DropdownUiBlock):
                children_item = children_item_data.to_dict()

            elif isinstance(children_item_data, MarkdownUiBlock):
                children_item = children_item_data.to_dict()

            elif isinstance(children_item_data, SearchInputUiBlock):
                children_item = children_item_data.to_dict()

            elif isinstance(children_item_data, SelectorInputUiBlock):
                children_item = children_item_data.to_dict()

            elif isinstance(children_item_data, TextInputUiBlock):
                children_item = children_item_data.to_dict()

            else:
                children_item = children_item_data.to_dict()

            children.append(children_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if children is not UNSET:
            field_dict["children"] = children

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_children() -> List[
            Union[
                ButtonUiBlock,
                ChipUiBlock,
                DropdownUiBlock,
                MarkdownUiBlock,
                SearchInputUiBlock,
                SelectorInputUiBlock,
                TextInputUiBlock,
                TextLabelUiBlock,
                UnknownType,
            ]
        ]:
            children = []
            _children = d.pop("children")
            for children_item_data in _children:

                def _parse_children_item(
                    data: Union[Dict[str, Any]]
                ) -> Union[
                    ButtonUiBlock,
                    ChipUiBlock,
                    DropdownUiBlock,
                    MarkdownUiBlock,
                    SearchInputUiBlock,
                    SelectorInputUiBlock,
                    TextInputUiBlock,
                    TextLabelUiBlock,
                    UnknownType,
                ]:
                    children_item: Union[
                        ButtonUiBlock,
                        ChipUiBlock,
                        DropdownUiBlock,
                        MarkdownUiBlock,
                        SearchInputUiBlock,
                        SelectorInputUiBlock,
                        TextInputUiBlock,
                        TextLabelUiBlock,
                        UnknownType,
                    ]
                    discriminator_value: str = cast(str, data.get("type"))
                    if discriminator_value is not None:
                        if discriminator_value == "BUTTON":
                            children_item = ButtonUiBlock.from_dict(data, strict=False)

                            return children_item
                        if discriminator_value == "CHIP":
                            children_item = ChipUiBlock.from_dict(data, strict=False)

                            return children_item
                        if discriminator_value == "DROPDOWN":
                            children_item = DropdownUiBlock.from_dict(data, strict=False)

                            return children_item
                        if discriminator_value == "LABEL":
                            children_item = TextLabelUiBlock.from_dict(data, strict=False)

                            return children_item
                        if discriminator_value == "MARKDOWN":
                            children_item = MarkdownUiBlock.from_dict(data, strict=False)

                            return children_item
                        if discriminator_value == "SEARCH_INPUT":
                            children_item = SearchInputUiBlock.from_dict(data, strict=False)

                            return children_item
                        if discriminator_value == "SELECTOR_INPUT":
                            children_item = SelectorInputUiBlock.from_dict(data, strict=False)

                            return children_item
                        if discriminator_value == "TEXT_INPUT":
                            children_item = TextInputUiBlock.from_dict(data, strict=False)

                            return children_item

                        return UnknownType(value=data)
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        children_item = ButtonUiBlock.from_dict(data, strict=True)

                        return children_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        children_item = ChipUiBlock.from_dict(data, strict=True)

                        return children_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        children_item = DropdownUiBlock.from_dict(data, strict=True)

                        return children_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        children_item = MarkdownUiBlock.from_dict(data, strict=True)

                        return children_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        children_item = SearchInputUiBlock.from_dict(data, strict=True)

                        return children_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        children_item = SelectorInputUiBlock.from_dict(data, strict=True)

                        return children_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        children_item = TextInputUiBlock.from_dict(data, strict=True)

                        return children_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        children_item = TextLabelUiBlock.from_dict(data, strict=True)

                        return children_item
                    except:  # noqa: E722
                        pass
                    return UnknownType(data)

                children_item = _parse_children_item(children_item_data)

                children.append(children_item)

            return children

        try:
            children = get_children()
        except KeyError:
            if strict:
                raise
            children = cast(
                List[
                    Union[
                        ButtonUiBlock,
                        ChipUiBlock,
                        DropdownUiBlock,
                        MarkdownUiBlock,
                        SearchInputUiBlock,
                        SelectorInputUiBlock,
                        TextInputUiBlock,
                        TextLabelUiBlock,
                        UnknownType,
                    ]
                ],
                UNSET,
            )

        canvas_leaf_node_ui_block_list = cls(
            children=children,
        )

        canvas_leaf_node_ui_block_list.additional_properties = d
        return canvas_leaf_node_ui_block_list

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def children(
        self,
    ) -> List[
        Union[
            ButtonUiBlock,
            ChipUiBlock,
            DropdownUiBlock,
            MarkdownUiBlock,
            SearchInputUiBlock,
            SelectorInputUiBlock,
            TextInputUiBlock,
            TextLabelUiBlock,
            UnknownType,
        ]
    ]:
        if isinstance(self._children, Unset):
            raise NotPresentError(self, "children")
        return self._children

    @children.setter
    def children(
        self,
        value: List[
            Union[
                ButtonUiBlock,
                ChipUiBlock,
                DropdownUiBlock,
                MarkdownUiBlock,
                SearchInputUiBlock,
                SelectorInputUiBlock,
                TextInputUiBlock,
                TextLabelUiBlock,
                UnknownType,
            ]
        ],
    ) -> None:
        self._children = value



import datetime
from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..extensions import NotPresentError, UnknownType
from ..models.checkout_record_status import CheckoutRecordStatus
from ..models.team_summary import TeamSummary
from ..models.user_summary import UserSummary
from ..types import UNSET, Unset

T = TypeVar("T", bound="CheckoutRecord")


@attr.s(auto_attribs=True, repr=False)
class CheckoutRecord:
    """
    *assignee field* is set if status is "RESERVED" or "CHECKED_OUT", or null if status is "AVAILABLE".

    *comment field* is set when container was last reserved, checked out, or checked into.

    *modifiedAt field* is the date and time when container was last checked out, checked in, or reserved
    """

    _assignee: Union[Unset, None, UserSummary, TeamSummary, UnknownType] = UNSET
    _comment: Union[Unset, str] = UNSET
    _modified_at: Union[Unset, datetime.datetime] = UNSET
    _status: Union[Unset, CheckoutRecordStatus] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("assignee={}".format(repr(self._assignee)))
        fields.append("comment={}".format(repr(self._comment)))
        fields.append("modified_at={}".format(repr(self._modified_at)))
        fields.append("status={}".format(repr(self._status)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "CheckoutRecord({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        assignee: Union[Unset, None, Dict[str, Any]]
        if isinstance(self._assignee, Unset):
            assignee = UNSET
        elif self._assignee is None:
            assignee = None
        elif isinstance(self._assignee, UnknownType):
            assignee = self._assignee.value
        elif isinstance(self._assignee, UserSummary):
            assignee = UNSET
            if not isinstance(self._assignee, Unset):
                assignee = self._assignee.to_dict()

        else:
            assignee = UNSET
            if not isinstance(self._assignee, Unset):
                assignee = self._assignee.to_dict()

        comment = self._comment
        modified_at: Union[Unset, str] = UNSET
        if not isinstance(self._modified_at, Unset):
            modified_at = self._modified_at.isoformat()

        status: Union[Unset, int] = UNSET
        if not isinstance(self._status, Unset):
            status = self._status.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if assignee is not UNSET:
            field_dict["assignee"] = assignee
        if comment is not UNSET:
            field_dict["comment"] = comment
        if modified_at is not UNSET:
            field_dict["modifiedAt"] = modified_at
        if status is not UNSET:
            field_dict["status"] = status

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_assignee() -> Union[Unset, None, UserSummary, TeamSummary, UnknownType]:
            def _parse_assignee(
                data: Union[Unset, None, Dict[str, Any]]
            ) -> Union[Unset, None, UserSummary, TeamSummary, UnknownType]:
                assignee: Union[Unset, None, UserSummary, TeamSummary, UnknownType]
                if data is None:
                    return data
                if isinstance(data, Unset):
                    return data
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    assignee = UNSET
                    _assignee = data

                    if not isinstance(_assignee, Unset):
                        assignee = UserSummary.from_dict(_assignee)

                    return assignee
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    assignee = UNSET
                    _assignee = data

                    if not isinstance(_assignee, Unset):
                        assignee = TeamSummary.from_dict(_assignee)

                    return assignee
                except:  # noqa: E722
                    pass
                return UnknownType(data)

            assignee = _parse_assignee(d.pop("assignee"))

            return assignee

        try:
            assignee = get_assignee()
        except KeyError:
            if strict:
                raise
            assignee = cast(Union[Unset, None, UserSummary, TeamSummary, UnknownType], UNSET)

        def get_comment() -> Union[Unset, str]:
            comment = d.pop("comment")
            return comment

        try:
            comment = get_comment()
        except KeyError:
            if strict:
                raise
            comment = cast(Union[Unset, str], UNSET)

        def get_modified_at() -> Union[Unset, datetime.datetime]:
            modified_at: Union[Unset, datetime.datetime] = UNSET
            _modified_at = d.pop("modifiedAt")
            if _modified_at is not None and not isinstance(_modified_at, Unset):
                modified_at = isoparse(cast(str, _modified_at))

            return modified_at

        try:
            modified_at = get_modified_at()
        except KeyError:
            if strict:
                raise
            modified_at = cast(Union[Unset, datetime.datetime], UNSET)

        def get_status() -> Union[Unset, CheckoutRecordStatus]:
            status = UNSET
            _status = d.pop("status")
            if _status is not None and _status is not UNSET:
                try:
                    status = CheckoutRecordStatus(_status)
                except ValueError:
                    status = CheckoutRecordStatus.of_unknown(_status)

            return status

        try:
            status = get_status()
        except KeyError:
            if strict:
                raise
            status = cast(Union[Unset, CheckoutRecordStatus], UNSET)

        checkout_record = cls(
            assignee=assignee,
            comment=comment,
            modified_at=modified_at,
            status=status,
        )

        checkout_record.additional_properties = d
        return checkout_record

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def assignee(self) -> Optional[Union[UserSummary, TeamSummary, UnknownType]]:
        if isinstance(self._assignee, Unset):
            raise NotPresentError(self, "assignee")
        return self._assignee

    @assignee.setter
    def assignee(self, value: Optional[Union[UserSummary, TeamSummary, UnknownType]]) -> None:
        self._assignee = value

    @assignee.deleter
    def assignee(self) -> None:
        self._assignee = UNSET

    @property
    def comment(self) -> str:
        if isinstance(self._comment, Unset):
            raise NotPresentError(self, "comment")
        return self._comment

    @comment.setter
    def comment(self, value: str) -> None:
        self._comment = value

    @comment.deleter
    def comment(self) -> None:
        self._comment = UNSET

    @property
    def modified_at(self) -> datetime.datetime:
        if isinstance(self._modified_at, Unset):
            raise NotPresentError(self, "modified_at")
        return self._modified_at

    @modified_at.setter
    def modified_at(self, value: datetime.datetime) -> None:
        self._modified_at = value

    @modified_at.deleter
    def modified_at(self) -> None:
        self._modified_at = UNSET

    @property
    def status(self) -> CheckoutRecordStatus:
        if isinstance(self._status, Unset):
            raise NotPresentError(self, "status")
        return self._status

    @status.setter
    def status(self, value: CheckoutRecordStatus) -> None:
        self._status = value

    @status.deleter
    def status(self) -> None:
        self._status = UNSET



from typing import Any, cast, Dict, List, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.boxes_archive_reason import BoxesArchiveReason
from ..types import UNSET, Unset

T = TypeVar("T", bound="BoxesArchive")


@attr.s(auto_attribs=True, repr=False)
class BoxesArchive:
    """  """

    _box_ids: List[str]
    _reason: BoxesArchiveReason
    _should_remove_barcodes: Union[Unset, bool] = UNSET

    def __repr__(self):
        fields = []
        fields.append("box_ids={}".format(repr(self._box_ids)))
        fields.append("reason={}".format(repr(self._reason)))
        fields.append("should_remove_barcodes={}".format(repr(self._should_remove_barcodes)))
        return "BoxesArchive({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        box_ids = self._box_ids

        reason = self._reason.value

        should_remove_barcodes = self._should_remove_barcodes

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if box_ids is not UNSET:
            field_dict["boxIds"] = box_ids
        if reason is not UNSET:
            field_dict["reason"] = reason
        if should_remove_barcodes is not UNSET:
            field_dict["shouldRemoveBarcodes"] = should_remove_barcodes

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_box_ids() -> List[str]:
            box_ids = cast(List[str], d.pop("boxIds"))

            return box_ids

        try:
            box_ids = get_box_ids()
        except KeyError:
            if strict:
                raise
            box_ids = cast(List[str], UNSET)

        def get_reason() -> BoxesArchiveReason:
            _reason = d.pop("reason")
            try:
                reason = BoxesArchiveReason(_reason)
            except ValueError:
                reason = BoxesArchiveReason.of_unknown(_reason)

            return reason

        try:
            reason = get_reason()
        except KeyError:
            if strict:
                raise
            reason = cast(BoxesArchiveReason, UNSET)

        def get_should_remove_barcodes() -> Union[Unset, bool]:
            should_remove_barcodes = d.pop("shouldRemoveBarcodes")
            return should_remove_barcodes

        try:
            should_remove_barcodes = get_should_remove_barcodes()
        except KeyError:
            if strict:
                raise
            should_remove_barcodes = cast(Union[Unset, bool], UNSET)

        boxes_archive = cls(
            box_ids=box_ids,
            reason=reason,
            should_remove_barcodes=should_remove_barcodes,
        )

        return boxes_archive

    @property
    def box_ids(self) -> List[str]:
        """ Array of box IDs """
        if isinstance(self._box_ids, Unset):
            raise NotPresentError(self, "box_ids")
        return self._box_ids

    @box_ids.setter
    def box_ids(self, value: List[str]) -> None:
        self._box_ids = value

    @property
    def reason(self) -> BoxesArchiveReason:
        """Reason that boxes are being archived."""
        if isinstance(self._reason, Unset):
            raise NotPresentError(self, "reason")
        return self._reason

    @reason.setter
    def reason(self, value: BoxesArchiveReason) -> None:
        self._reason = value

    @property
    def should_remove_barcodes(self) -> bool:
        """Remove barcodes. Removing barcodes from archived inventory that contain items will also remove barcodes from the contained items."""
        if isinstance(self._should_remove_barcodes, Unset):
            raise NotPresentError(self, "should_remove_barcodes")
        return self._should_remove_barcodes

    @should_remove_barcodes.setter
    def should_remove_barcodes(self, value: bool) -> None:
        self._should_remove_barcodes = value

    @should_remove_barcodes.deleter
    def should_remove_barcodes(self) -> None:
        self._should_remove_barcodes = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.dna_sequence_summary_entity_type import DnaSequenceSummaryEntityType
from ..types import UNSET, Unset

T = TypeVar("T", bound="DnaSequenceSummary")


@attr.s(auto_attribs=True, repr=False)
class DnaSequenceSummary:
    """  """

    _entity_type: Union[Unset, DnaSequenceSummaryEntityType] = UNSET
    _id: Union[Unset, str] = UNSET
    _type: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("entity_type={}".format(repr(self._entity_type)))
        fields.append("id={}".format(repr(self._id)))
        fields.append("type={}".format(repr(self._type)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "DnaSequenceSummary({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        entity_type: Union[Unset, int] = UNSET
        if not isinstance(self._entity_type, Unset):
            entity_type = self._entity_type.value

        id = self._id
        type = self._type

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if entity_type is not UNSET:
            field_dict["entityType"] = entity_type
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_entity_type() -> Union[Unset, DnaSequenceSummaryEntityType]:
            entity_type = UNSET
            _entity_type = d.pop("entityType")
            if _entity_type is not None and _entity_type is not UNSET:
                try:
                    entity_type = DnaSequenceSummaryEntityType(_entity_type)
                except ValueError:
                    entity_type = DnaSequenceSummaryEntityType.of_unknown(_entity_type)

            return entity_type

        try:
            entity_type = get_entity_type()
        except KeyError:
            if strict:
                raise
            entity_type = cast(Union[Unset, DnaSequenceSummaryEntityType], UNSET)

        def get_id() -> Union[Unset, str]:
            id = d.pop("id")
            return id

        try:
            id = get_id()
        except KeyError:
            if strict:
                raise
            id = cast(Union[Unset, str], UNSET)

        def get_type() -> Union[Unset, str]:
            type = d.pop("type")
            return type

        try:
            type = get_type()
        except KeyError:
            if strict:
                raise
            type = cast(Union[Unset, str], UNSET)

        dna_sequence_summary = cls(
            entity_type=entity_type,
            id=id,
            type=type,
        )

        dna_sequence_summary.additional_properties = d
        return dna_sequence_summary

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def entity_type(self) -> DnaSequenceSummaryEntityType:
        if isinstance(self._entity_type, Unset):
            raise NotPresentError(self, "entity_type")
        return self._entity_type

    @entity_type.setter
    def entity_type(self, value: DnaSequenceSummaryEntityType) -> None:
        self._entity_type = value

    @entity_type.deleter
    def entity_type(self) -> None:
        self._entity_type = UNSET

    @property
    def id(self) -> str:
        if isinstance(self._id, Unset):
            raise NotPresentError(self, "id")
        return self._id

    @id.setter
    def id(self, value: str) -> None:
        self._id = value

    @id.deleter
    def id(self) -> None:
        self._id = UNSET

    @property
    def type(self) -> str:
        if isinstance(self._type, Unset):
            raise NotPresentError(self, "type")
        return self._type

    @type.setter
    def type(self, value: str) -> None:
        self._type = value

    @type.deleter
    def type(self) -> None:
        self._type = UNSET



from typing import Any, cast, Dict, List, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..types import UNSET, Unset

T = TypeVar("T", bound="Enzyme")


@attr.s(auto_attribs=True, repr=False)
class Enzyme:
    """  """

    _cutsites: Union[Unset, List[int]] = UNSET
    _id: Union[Unset, str] = UNSET
    _isoschizomers: Union[Unset, List[str]] = UNSET
    _name: Union[Unset, str] = UNSET
    _offsets: Union[Unset, List[int]] = UNSET
    _restriction_site: Union[Unset, str] = UNSET

    def __repr__(self):
        fields = []
        fields.append("cutsites={}".format(repr(self._cutsites)))
        fields.append("id={}".format(repr(self._id)))
        fields.append("isoschizomers={}".format(repr(self._isoschizomers)))
        fields.append("name={}".format(repr(self._name)))
        fields.append("offsets={}".format(repr(self._offsets)))
        fields.append("restriction_site={}".format(repr(self._restriction_site)))
        return "Enzyme({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        cutsites: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._cutsites, Unset):
            cutsites = self._cutsites

        id = self._id
        isoschizomers: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._isoschizomers, Unset):
            isoschizomers = self._isoschizomers

        name = self._name
        offsets: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._offsets, Unset):
            offsets = self._offsets

        restriction_site = self._restriction_site

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if cutsites is not UNSET:
            field_dict["cutsites"] = cutsites
        if id is not UNSET:
            field_dict["id"] = id
        if isoschizomers is not UNSET:
            field_dict["isoschizomers"] = isoschizomers
        if name is not UNSET:
            field_dict["name"] = name
        if offsets is not UNSET:
            field_dict["offsets"] = offsets
        if restriction_site is not UNSET:
            field_dict["restrictionSite"] = restriction_site

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_cutsites() -> Union[Unset, List[int]]:
            cutsites = cast(List[int], d.pop("cutsites"))

            return cutsites

        try:
            cutsites = get_cutsites()
        except KeyError:
            if strict:
                raise
            cutsites = cast(Union[Unset, List[int]], UNSET)

        def get_id() -> Union[Unset, str]:
            id = d.pop("id")
            return id

        try:
            id = get_id()
        except KeyError:
            if strict:
                raise
            id = cast(Union[Unset, str], UNSET)

        def get_isoschizomers() -> Union[Unset, List[str]]:
            isoschizomers = cast(List[str], d.pop("isoschizomers"))

            return isoschizomers

        try:
            isoschizomers = get_isoschizomers()
        except KeyError:
            if strict:
                raise
            isoschizomers = cast(Union[Unset, List[str]], UNSET)

        def get_name() -> Union[Unset, str]:
            name = d.pop("name")
            return name

        try:
            name = get_name()
        except KeyError:
            if strict:
                raise
            name = cast(Union[Unset, str], UNSET)

        def get_offsets() -> Union[Unset, List[int]]:
            offsets = cast(List[int], d.pop("offsets"))

            return offsets

        try:
            offsets = get_offsets()
        except KeyError:
            if strict:
                raise
            offsets = cast(Union[Unset, List[int]], UNSET)

        def get_restriction_site() -> Union[Unset, str]:
            restriction_site = d.pop("restrictionSite")
            return restriction_site

        try:
            restriction_site = get_restriction_site()
        except KeyError:
            if strict:
                raise
            restriction_site = cast(Union[Unset, str], UNSET)

        enzyme = cls(
            cutsites=cutsites,
            id=id,
            isoschizomers=isoschizomers,
            name=name,
            offsets=offsets,
            restriction_site=restriction_site,
        )

        return enzyme

    @property
    def cutsites(self) -> List[int]:
        if isinstance(self._cutsites, Unset):
            raise NotPresentError(self, "cutsites")
        return self._cutsites

    @cutsites.setter
    def cutsites(self, value: List[int]) -> None:
        self._cutsites = value

    @cutsites.deleter
    def cutsites(self) -> None:
        self._cutsites = UNSET

    @property
    def id(self) -> str:
        if isinstance(self._id, Unset):
            raise NotPresentError(self, "id")
        return self._id

    @id.setter
    def id(self, value: str) -> None:
        self._id = value

    @id.deleter
    def id(self) -> None:
        self._id = UNSET

    @property
    def isoschizomers(self) -> List[str]:
        if isinstance(self._isoschizomers, Unset):
            raise NotPresentError(self, "isoschizomers")
        return self._isoschizomers

    @isoschizomers.setter
    def isoschizomers(self, value: List[str]) -> None:
        self._isoschizomers = value

    @isoschizomers.deleter
    def isoschizomers(self) -> None:
        self._isoschizomers = UNSET

    @property
    def name(self) -> str:
        if isinstance(self._name, Unset):
            raise NotPresentError(self, "name")
        return self._name

    @name.setter
    def name(self, value: str) -> None:
        self._name = value

    @name.deleter
    def name(self) -> None:
        self._name = UNSET

    @property
    def offsets(self) -> List[int]:
        if isinstance(self._offsets, Unset):
            raise NotPresentError(self, "offsets")
        return self._offsets

    @offsets.setter
    def offsets(self, value: List[int]) -> None:
        self._offsets = value

    @offsets.deleter
    def offsets(self) -> None:
        self._offsets = UNSET

    @property
    def restriction_site(self) -> str:
        if isinstance(self._restriction_site, Unset):
            raise NotPresentError(self, "restriction_site")
        return self._restriction_site

    @restriction_site.setter
    def restriction_site(self, value: str) -> None:
        self._restriction_site = value

    @restriction_site.deleter
    def restriction_site(self) -> None:
        self._restriction_site = UNSET



from enum import Enum
from functools import lru_cache
from typing import cast

from ..extensions import Enums


class IngredientMeasurementUnits(Enums.KnownString):
    NL = "nL"
    UL = "uL"
    ML = "mL"
    L = "L"
    MG = "mg"
    G = "g"
    UNITS = "Units"

    def __str__(self) -> str:
        return str(self.value)

    @staticmethod
    @lru_cache(maxsize=None)
    def of_unknown(val: str) -> "IngredientMeasurementUnits":
        if not isinstance(val, str):
            raise ValueError(f"Value of IngredientMeasurementUnits must be a string (encountered: {val})")
        newcls = Enum("IngredientMeasurementUnits", {"_UNKNOWN": val}, type=Enums.UnknownString)  # type: ignore
        return cast(IngredientMeasurementUnits, getattr(newcls, "_UNKNOWN"))



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.label_template import LabelTemplate
from ..types import UNSET, Unset

T = TypeVar("T", bound="LabelTemplatesList")


@attr.s(auto_attribs=True, repr=False)
class LabelTemplatesList:
    """  """

    _label_templates: Union[Unset, List[LabelTemplate]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("label_templates={}".format(repr(self._label_templates)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "LabelTemplatesList({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        label_templates: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._label_templates, Unset):
            label_templates = []
            for label_templates_item_data in self._label_templates:
                label_templates_item = label_templates_item_data.to_dict()

                label_templates.append(label_templates_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if label_templates is not UNSET:
            field_dict["labelTemplates"] = label_templates

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_label_templates() -> Union[Unset, List[LabelTemplate]]:
            label_templates = []
            _label_templates = d.pop("labelTemplates")
            for label_templates_item_data in _label_templates or []:
                label_templates_item = LabelTemplate.from_dict(label_templates_item_data, strict=False)

                label_templates.append(label_templates_item)

            return label_templates

        try:
            label_templates = get_label_templates()
        except KeyError:
            if strict:
                raise
            label_templates = cast(Union[Unset, List[LabelTemplate]], UNSET)

        label_templates_list = cls(
            label_templates=label_templates,
        )

        label_templates_list.additional_properties = d
        return label_templates_list

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def label_templates(self) -> List[LabelTemplate]:
        if isinstance(self._label_templates, Unset):
            raise NotPresentError(self, "label_templates")
        return self._label_templates

    @label_templates.setter
    def label_templates(self, value: List[LabelTemplate]) -> None:
        self._label_templates = value

    @label_templates.deleter
    def label_templates(self) -> None:
        self._label_templates = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.workflow_task_status_status_type import WorkflowTaskStatusStatusType
from ..types import UNSET, Unset

T = TypeVar("T", bound="WorkflowTaskStatus")


@attr.s(auto_attribs=True, repr=False)
class WorkflowTaskStatus:
    """  """

    _display_name: Union[Unset, str] = UNSET
    _id: Union[Unset, str] = UNSET
    _status_type: Union[Unset, WorkflowTaskStatusStatusType] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("display_name={}".format(repr(self._display_name)))
        fields.append("id={}".format(repr(self._id)))
        fields.append("status_type={}".format(repr(self._status_type)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "WorkflowTaskStatus({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        display_name = self._display_name
        id = self._id
        status_type: Union[Unset, int] = UNSET
        if not isinstance(self._status_type, Unset):
            status_type = self._status_type.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if display_name is not UNSET:
            field_dict["displayName"] = display_name
        if id is not UNSET:
            field_dict["id"] = id
        if status_type is not UNSET:
            field_dict["statusType"] = status_type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_display_name() -> Union[Unset, str]:
            display_name = d.pop("displayName")
            return display_name

        try:
            display_name = get_display_name()
        except KeyError:
            if strict:
                raise
            display_name = cast(Union[Unset, str], UNSET)

        def get_id() -> Union[Unset, str]:
            id = d.pop("id")
            return id

        try:
            id = get_id()
        except KeyError:
            if strict:
                raise
            id = cast(Union[Unset, str], UNSET)

        def get_status_type() -> Union[Unset, WorkflowTaskStatusStatusType]:
            status_type = UNSET
            _status_type = d.pop("statusType")
            if _status_type is not None and _status_type is not UNSET:
                try:
                    status_type = WorkflowTaskStatusStatusType(_status_type)
                except ValueError:
                    status_type = WorkflowTaskStatusStatusType.of_unknown(_status_type)

            return status_type

        try:
            status_type = get_status_type()
        except KeyError:
            if strict:
                raise
            status_type = cast(Union[Unset, WorkflowTaskStatusStatusType], UNSET)

        workflow_task_status = cls(
            display_name=display_name,
            id=id,
            status_type=status_type,
        )

        workflow_task_status.additional_properties = d
        return workflow_task_status

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def display_name(self) -> str:
        """ The status label """
        if isinstance(self._display_name, Unset):
            raise NotPresentError(self, "display_name")
        return self._display_name

    @display_name.setter
    def display_name(self, value: str) -> None:
        self._display_name = value

    @display_name.deleter
    def display_name(self) -> None:
        self._display_name = UNSET

    @property
    def id(self) -> str:
        """ The ID of the workflow task status """
        if isinstance(self._id, Unset):
            raise NotPresentError(self, "id")
        return self._id

    @id.setter
    def id(self, value: str) -> None:
        self._id = value

    @id.deleter
    def id(self) -> None:
        self._id = UNSET

    @property
    def status_type(self) -> WorkflowTaskStatusStatusType:
        """ The status type """
        if isinstance(self._status_type, Unset):
            raise NotPresentError(self, "status_type")
        return self._status_type

    @status_type.setter
    def status_type(self, value: WorkflowTaskStatusStatusType) -> None:
        self._status_type = value

    @status_type.deleter
    def status_type(self) -> None:
        self._status_type = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.analysis_step_dataset_data_kind import AnalysisStepDatasetDataKind
from ..types import UNSET, Unset

T = TypeVar("T", bound="AnalysisStepDatasetData")


@attr.s(auto_attribs=True, repr=False)
class AnalysisStepDatasetData:
    """  """

    _id: Union[Unset, None] = UNSET
    _is_multi: Union[Unset, None] = UNSET
    _kind: Union[Unset, AnalysisStepDatasetDataKind] = UNSET
    _name: Union[Unset, None] = UNSET
    _dataset_ids: Union[Unset, List[str]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("id={}".format(repr(self._id)))
        fields.append("is_multi={}".format(repr(self._is_multi)))
        fields.append("kind={}".format(repr(self._kind)))
        fields.append("name={}".format(repr(self._name)))
        fields.append("dataset_ids={}".format(repr(self._dataset_ids)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "AnalysisStepDatasetData({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        id = None

        is_multi = None

        kind: Union[Unset, int] = UNSET
        if not isinstance(self._kind, Unset):
            kind = self._kind.value

        name = None

        dataset_ids: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._dataset_ids, Unset):
            dataset_ids = self._dataset_ids

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if id is not UNSET:
            field_dict["id"] = id
        if is_multi is not UNSET:
            field_dict["isMulti"] = is_multi
        if kind is not UNSET:
            field_dict["kind"] = kind
        if name is not UNSET:
            field_dict["name"] = name
        if dataset_ids is not UNSET:
            field_dict["datasetIds"] = dataset_ids

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_id() -> Union[Unset, None]:
            id = None

            return id

        try:
            id = get_id()
        except KeyError:
            if strict:
                raise
            id = cast(Union[Unset, None], UNSET)

        def get_is_multi() -> Union[Unset, None]:
            is_multi = None

            return is_multi

        try:
            is_multi = get_is_multi()
        except KeyError:
            if strict:
                raise
            is_multi = cast(Union[Unset, None], UNSET)

        def get_kind() -> Union[Unset, AnalysisStepDatasetDataKind]:
            kind = UNSET
            _kind = d.pop("kind")
            if _kind is not None and _kind is not UNSET:
                try:
                    kind = AnalysisStepDatasetDataKind(_kind)
                except ValueError:
                    kind = AnalysisStepDatasetDataKind.of_unknown(_kind)

            return kind

        try:
            kind = get_kind()
        except KeyError:
            if strict:
                raise
            kind = cast(Union[Unset, AnalysisStepDatasetDataKind], UNSET)

        def get_name() -> Union[Unset, None]:
            name = None

            return name

        try:
            name = get_name()
        except KeyError:
            if strict:
                raise
            name = cast(Union[Unset, None], UNSET)

        def get_dataset_ids() -> Union[Unset, List[str]]:
            dataset_ids = cast(List[str], d.pop("datasetIds"))

            return dataset_ids

        try:
            dataset_ids = get_dataset_ids()
        except KeyError:
            if strict:
                raise
            dataset_ids = cast(Union[Unset, List[str]], UNSET)

        analysis_step_dataset_data = cls(
            id=id,
            is_multi=is_multi,
            kind=kind,
            name=name,
            dataset_ids=dataset_ids,
        )

        analysis_step_dataset_data.additional_properties = d
        return analysis_step_dataset_data

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def id(self) -> None:
        if isinstance(self._id, Unset):
            raise NotPresentError(self, "id")
        return self._id

    @id.setter
    def id(self, value: None) -> None:
        self._id = value

    @id.deleter
    def id(self) -> None:
        self._id = UNSET

    @property
    def is_multi(self) -> None:
        if isinstance(self._is_multi, Unset):
            raise NotPresentError(self, "is_multi")
        return self._is_multi

    @is_multi.setter
    def is_multi(self, value: None) -> None:
        self._is_multi = value

    @is_multi.deleter
    def is_multi(self) -> None:
        self._is_multi = UNSET

    @property
    def kind(self) -> AnalysisStepDatasetDataKind:
        if isinstance(self._kind, Unset):
            raise NotPresentError(self, "kind")
        return self._kind

    @kind.setter
    def kind(self, value: AnalysisStepDatasetDataKind) -> None:
        self._kind = value

    @kind.deleter
    def kind(self) -> None:
        self._kind = UNSET

    @property
    def name(self) -> None:
        if isinstance(self._name, Unset):
            raise NotPresentError(self, "name")
        return self._name

    @name.setter
    def name(self, value: None) -> None:
        self._name = value

    @name.deleter
    def name(self) -> None:
        self._name = UNSET

    @property
    def dataset_ids(self) -> List[str]:
        if isinstance(self._dataset_ids, Unset):
            raise NotPresentError(self, "dataset_ids")
        return self._dataset_ids

    @dataset_ids.setter
    def dataset_ids(self, value: List[str]) -> None:
        self._dataset_ids = value

    @dataset_ids.deleter
    def dataset_ids(self) -> None:
        self._dataset_ids = UNSET



import datetime
from typing import Any, cast, Dict, Optional, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..extensions import NotPresentError
from ..models.archive_record import ArchiveRecord
from ..models.user_summary import UserSummary
from ..types import UNSET, Unset

T = TypeVar("T", bound="BenchlingApp")


@attr.s(auto_attribs=True, repr=False)
class BenchlingApp:
    """  """

    _api_url: Union[Unset, str] = UNSET
    _archive_record: Union[Unset, None, ArchiveRecord] = UNSET
    _created_at: Union[Unset, datetime.datetime] = UNSET
    _creator: Union[Unset, UserSummary] = UNSET
    _id: Union[Unset, str] = UNSET
    _modified_at: Union[Unset, datetime.datetime] = UNSET
    _web_url: Union[Unset, str] = UNSET
    _description: Union[Unset, str] = UNSET
    _name: Union[Unset, str] = UNSET

    def __repr__(self):
        fields = []
        fields.append("api_url={}".format(repr(self._api_url)))
        fields.append("archive_record={}".format(repr(self._archive_record)))
        fields.append("created_at={}".format(repr(self._created_at)))
        fields.append("creator={}".format(repr(self._creator)))
        fields.append("id={}".format(repr(self._id)))
        fields.append("modified_at={}".format(repr(self._modified_at)))
        fields.append("web_url={}".format(repr(self._web_url)))
        fields.append("description={}".format(repr(self._description)))
        fields.append("name={}".format(repr(self._name)))
        return "BenchlingApp({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        api_url = self._api_url
        archive_record: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self._archive_record, Unset):
            archive_record = self._archive_record.to_dict() if self._archive_record else None

        created_at: Union[Unset, str] = UNSET
        if not isinstance(self._created_at, Unset):
            created_at = self._created_at.isoformat()

        creator: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._creator, Unset):
            creator = self._creator.to_dict()

        id = self._id
        modified_at: Union[Unset, str] = UNSET
        if not isinstance(self._modified_at, Unset):
            modified_at = self._modified_at.isoformat()

        web_url = self._web_url
        description = self._description
        name = self._name

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if api_url is not UNSET:
            field_dict["apiUrl"] = api_url
        if archive_record is not UNSET:
            field_dict["archiveRecord"] = archive_record
        if created_at is not UNSET:
            field_dict["createdAt"] = created_at
        if creator is not UNSET:
            field_dict["creator"] = creator
        if id is not UNSET:
            field_dict["id"] = id
        if modified_at is not UNSET:
            field_dict["modifiedAt"] = modified_at
        if web_url is not UNSET:
            field_dict["webUrl"] = web_url
        if description is not UNSET:
            field_dict["description"] = description
        if name is not UNSET:
            field_dict["name"] = name

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_api_url() -> Union[Unset, str]:
            api_url = d.pop("apiUrl")
            return api_url

        try:
            api_url = get_api_url()
        except KeyError:
            if strict:
                raise
            api_url = cast(Union[Unset, str], UNSET)

        def get_archive_record() -> Union[Unset, None, ArchiveRecord]:
            archive_record = None
            _archive_record = d.pop("archiveRecord")

            if _archive_record is not None and not isinstance(_archive_record, Unset):
                archive_record = ArchiveRecord.from_dict(_archive_record)

            return archive_record

        try:
            archive_record = get_archive_record()
        except KeyError:
            if strict:
                raise
            archive_record = cast(Union[Unset, None, ArchiveRecord], UNSET)

        def get_created_at() -> Union[Unset, datetime.datetime]:
            created_at: Union[Unset, datetime.datetime] = UNSET
            _created_at = d.pop("createdAt")
            if _created_at is not None and not isinstance(_created_at, Unset):
                created_at = isoparse(cast(str, _created_at))

            return created_at

        try:
            created_at = get_created_at()
        except KeyError:
            if strict:
                raise
            created_at = cast(Union[Unset, datetime.datetime], UNSET)

        def get_creator() -> Union[Unset, UserSummary]:
            creator: Union[Unset, Union[Unset, UserSummary]] = UNSET
            _creator = d.pop("creator")

            if not isinstance(_creator, Unset):
                creator = UserSummary.from_dict(_creator)

            return creator

        try:
            creator = get_creator()
        except KeyError:
            if strict:
                raise
            creator = cast(Union[Unset, UserSummary], UNSET)

        def get_id() -> Union[Unset, str]:
            id = d.pop("id")
            return id

        try:
            id = get_id()
        except KeyError:
            if strict:
                raise
            id = cast(Union[Unset, str], UNSET)

        def get_modified_at() -> Union[Unset, datetime.datetime]:
            modified_at: Union[Unset, datetime.datetime] = UNSET
            _modified_at = d.pop("modifiedAt")
            if _modified_at is not None and not isinstance(_modified_at, Unset):
                modified_at = isoparse(cast(str, _modified_at))

            return modified_at

        try:
            modified_at = get_modified_at()
        except KeyError:
            if strict:
                raise
            modified_at = cast(Union[Unset, datetime.datetime], UNSET)

        def get_web_url() -> Union[Unset, str]:
            web_url = d.pop("webUrl")
            return web_url

        try:
            web_url = get_web_url()
        except KeyError:
            if strict:
                raise
            web_url = cast(Union[Unset, str], UNSET)

        def get_description() -> Union[Unset, str]:
            description = d.pop("description")
            return description

        try:
            description = get_description()
        except KeyError:
            if strict:
                raise
            description = cast(Union[Unset, str], UNSET)

        def get_name() -> Union[Unset, str]:
            name = d.pop("name")
            return name

        try:
            name = get_name()
        except KeyError:
            if strict:
                raise
            name = cast(Union[Unset, str], UNSET)

        benchling_app = cls(
            api_url=api_url,
            archive_record=archive_record,
            created_at=created_at,
            creator=creator,
            id=id,
            modified_at=modified_at,
            web_url=web_url,
            description=description,
            name=name,
        )

        return benchling_app

    @property
    def api_url(self) -> str:
        if isinstance(self._api_url, Unset):
            raise NotPresentError(self, "api_url")
        return self._api_url

    @api_url.setter
    def api_url(self, value: str) -> None:
        self._api_url = value

    @api_url.deleter
    def api_url(self) -> None:
        self._api_url = UNSET

    @property
    def archive_record(self) -> Optional[ArchiveRecord]:
        if isinstance(self._archive_record, Unset):
            raise NotPresentError(self, "archive_record")
        return self._archive_record

    @archive_record.setter
    def archive_record(self, value: Optional[ArchiveRecord]) -> None:
        self._archive_record = value

    @archive_record.deleter
    def archive_record(self) -> None:
        self._archive_record = UNSET

    @property
    def created_at(self) -> datetime.datetime:
        """ DateTime at which the the app was created """
        if isinstance(self._created_at, Unset):
            raise NotPresentError(self, "created_at")
        return self._created_at

    @created_at.setter
    def created_at(self, value: datetime.datetime) -> None:
        self._created_at = value

    @created_at.deleter
    def created_at(self) -> None:
        self._created_at = UNSET

    @property
    def creator(self) -> UserSummary:
        if isinstance(self._creator, Unset):
            raise NotPresentError(self, "creator")
        return self._creator

    @creator.setter
    def creator(self, value: UserSummary) -> None:
        self._creator = value

    @creator.deleter
    def creator(self) -> None:
        self._creator = UNSET

    @property
    def id(self) -> str:
        if isinstance(self._id, Unset):
            raise NotPresentError(self, "id")
        return self._id

    @id.setter
    def id(self, value: str) -> None:
        self._id = value

    @id.deleter
    def id(self) -> None:
        self._id = UNSET

    @property
    def modified_at(self) -> datetime.datetime:
        """ DateTime at which the the app was last modified """
        if isinstance(self._modified_at, Unset):
            raise NotPresentError(self, "modified_at")
        return self._modified_at

    @modified_at.setter
    def modified_at(self, value: datetime.datetime) -> None:
        self._modified_at = value

    @modified_at.deleter
    def modified_at(self) -> None:
        self._modified_at = UNSET

    @property
    def web_url(self) -> str:
        if isinstance(self._web_url, Unset):
            raise NotPresentError(self, "web_url")
        return self._web_url

    @web_url.setter
    def web_url(self, value: str) -> None:
        self._web_url = value

    @web_url.deleter
    def web_url(self) -> None:
        self._web_url = UNSET

    @property
    def description(self) -> str:
        if isinstance(self._description, Unset):
            raise NotPresentError(self, "description")
        return self._description

    @description.setter
    def description(self, value: str) -> None:
        self._description = value

    @description.deleter
    def description(self) -> None:
        self._description = UNSET

    @property
    def name(self) -> str:
        if isinstance(self._name, Unset):
            raise NotPresentError(self, "name")
        return self._name

    @name.setter
    def name(self, value: str) -> None:
        self._name = value

    @name.deleter
    def name(self) -> None:
        self._name = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..types import UNSET, Unset

T = TypeVar("T", bound="EntitySchemaConstraint")


@attr.s(auto_attribs=True, repr=False)
class EntitySchemaConstraint:
    """  """

    _field_definition_names: Union[Unset, List[str]] = UNSET
    _has_unique_residues: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("field_definition_names={}".format(repr(self._field_definition_names)))
        fields.append("has_unique_residues={}".format(repr(self._has_unique_residues)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "EntitySchemaConstraint({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        field_definition_names: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._field_definition_names, Unset):
            field_definition_names = self._field_definition_names

        has_unique_residues = self._has_unique_residues

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if field_definition_names is not UNSET:
            field_dict["fieldDefinitionNames"] = field_definition_names
        if has_unique_residues is not UNSET:
            field_dict["hasUniqueResidues"] = has_unique_residues

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_field_definition_names() -> Union[Unset, List[str]]:
            field_definition_names = cast(List[str], d.pop("fieldDefinitionNames"))

            return field_definition_names

        try:
            field_definition_names = get_field_definition_names()
        except KeyError:
            if strict:
                raise
            field_definition_names = cast(Union[Unset, List[str]], UNSET)

        def get_has_unique_residues() -> Union[Unset, bool]:
            has_unique_residues = d.pop("hasUniqueResidues")
            return has_unique_residues

        try:
            has_unique_residues = get_has_unique_residues()
        except KeyError:
            if strict:
                raise
            has_unique_residues = cast(Union[Unset, bool], UNSET)

        entity_schema_constraint = cls(
            field_definition_names=field_definition_names,
            has_unique_residues=has_unique_residues,
        )

        entity_schema_constraint.additional_properties = d
        return entity_schema_constraint

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def field_definition_names(self) -> List[str]:
        if isinstance(self._field_definition_names, Unset):
            raise NotPresentError(self, "field_definition_names")
        return self._field_definition_names

    @field_definition_names.setter
    def field_definition_names(self, value: List[str]) -> None:
        self._field_definition_names = value

    @field_definition_names.deleter
    def field_definition_names(self) -> None:
        self._field_definition_names = UNSET

    @property
    def has_unique_residues(self) -> bool:
        if isinstance(self._has_unique_residues, Unset):
            raise NotPresentError(self, "has_unique_residues")
        return self._has_unique_residues

    @has_unique_residues.setter
    def has_unique_residues(self, value: bool) -> None:
        self._has_unique_residues = value

    @has_unique_residues.deleter
    def has_unique_residues(self) -> None:
        self._has_unique_residues = UNSET



from typing import Any, cast, Dict, List, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.enzyme import Enzyme
from ..types import UNSET, Unset

T = TypeVar("T", bound="EnzymesPaginatedList")


@attr.s(auto_attribs=True, repr=False)
class EnzymesPaginatedList:
    """  """

    _enzymes: Union[Unset, List[Enzyme]] = UNSET
    _next_token: Union[Unset, str] = UNSET

    def __repr__(self):
        fields = []
        fields.append("enzymes={}".format(repr(self._enzymes)))
        fields.append("next_token={}".format(repr(self._next_token)))
        return "EnzymesPaginatedList({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        enzymes: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._enzymes, Unset):
            enzymes = []
            for enzymes_item_data in self._enzymes:
                enzymes_item = enzymes_item_data.to_dict()

                enzymes.append(enzymes_item)

        next_token = self._next_token

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if enzymes is not UNSET:
            field_dict["enzymes"] = enzymes
        if next_token is not UNSET:
            field_dict["nextToken"] = next_token

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_enzymes() -> Union[Unset, List[Enzyme]]:
            enzymes = []
            _enzymes = d.pop("enzymes")
            for enzymes_item_data in _enzymes or []:
                enzymes_item = Enzyme.from_dict(enzymes_item_data, strict=False)

                enzymes.append(enzymes_item)

            return enzymes

        try:
            enzymes = get_enzymes()
        except KeyError:
            if strict:
                raise
            enzymes = cast(Union[Unset, List[Enzyme]], UNSET)

        def get_next_token() -> Union[Unset, str]:
            next_token = d.pop("nextToken")
            return next_token

        try:
            next_token = get_next_token()
        except KeyError:
            if strict:
                raise
            next_token = cast(Union[Unset, str], UNSET)

        enzymes_paginated_list = cls(
            enzymes=enzymes,
            next_token=next_token,
        )

        return enzymes_paginated_list

    @property
    def enzymes(self) -> List[Enzyme]:
        if isinstance(self._enzymes, Unset):
            raise NotPresentError(self, "enzymes")
        return self._enzymes

    @enzymes.setter
    def enzymes(self, value: List[Enzyme]) -> None:
        self._enzymes = value

    @enzymes.deleter
    def enzymes(self) -> None:
        self._enzymes = UNSET

    @property
    def next_token(self) -> str:
        if isinstance(self._next_token, Unset):
            raise NotPresentError(self, "next_token")
        return self._next_token

    @next_token.setter
    def next_token(self, value: str) -> None:
        self._next_token = value

    @next_token.deleter
    def next_token(self) -> None:
        self._next_token = UNSET



import datetime
from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..extensions import NotPresentError, UnknownType
from ..models.archive_record import ArchiveRecord
from ..models.deprecated_entity_schema_type import DeprecatedEntitySchemaType
from ..models.dropdown_field_definition import DropdownFieldDefinition
from ..models.entity_schema_constraint import EntitySchemaConstraint
from ..models.entity_schema_containable_type import EntitySchemaContainableType
from ..models.float_field_definition import FloatFieldDefinition
from ..models.integer_field_definition import IntegerFieldDefinition
from ..models.schema_link_field_definition import SchemaLinkFieldDefinition
from ..models.simple_field_definition import SimpleFieldDefinition
from ..types import UNSET, Unset

T = TypeVar("T", bound="DeprecatedEntitySchema")


@attr.s(auto_attribs=True, repr=False)
class DeprecatedEntitySchema:
    """  """

    _type: Union[Unset, DeprecatedEntitySchemaType] = UNSET
    _constraint: Union[Unset, None, EntitySchemaConstraint] = UNSET
    _containable_type: Union[Unset, EntitySchemaContainableType] = UNSET
    _modified_at: Union[Unset, datetime.datetime] = UNSET
    _prefix: Union[Unset, str] = UNSET
    _registry_id: Union[Unset, str] = UNSET
    _archive_record: Union[Unset, None, ArchiveRecord] = UNSET
    _field_definitions: Union[
        Unset,
        List[
            Union[
                SimpleFieldDefinition,
                IntegerFieldDefinition,
                FloatFieldDefinition,
                DropdownFieldDefinition,
                SchemaLinkFieldDefinition,
                UnknownType,
            ]
        ],
    ] = UNSET
    _id: Union[Unset, str] = UNSET
    _name: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("type={}".format(repr(self._type)))
        fields.append("constraint={}".format(repr(self._constraint)))
        fields.append("containable_type={}".format(repr(self._containable_type)))
        fields.append("modified_at={}".format(repr(self._modified_at)))
        fields.append("prefix={}".format(repr(self._prefix)))
        fields.append("registry_id={}".format(repr(self._registry_id)))
        fields.append("archive_record={}".format(repr(self._archive_record)))
        fields.append("field_definitions={}".format(repr(self._field_definitions)))
        fields.append("id={}".format(repr(self._id)))
        fields.append("name={}".format(repr(self._name)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "DeprecatedEntitySchema({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        type: Union[Unset, int] = UNSET
        if not isinstance(self._type, Unset):
            type = self._type.value

        constraint: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self._constraint, Unset):
            constraint = self._constraint.to_dict() if self._constraint else None

        containable_type: Union[Unset, int] = UNSET
        if not isinstance(self._containable_type, Unset):
            containable_type = self._containable_type.value

        modified_at: Union[Unset, str] = UNSET
        if not isinstance(self._modified_at, Unset):
            modified_at = self._modified_at.isoformat()

        prefix = self._prefix
        registry_id = self._registry_id
        archive_record: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self._archive_record, Unset):
            archive_record = self._archive_record.to_dict() if self._archive_record else None

        field_definitions: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._field_definitions, Unset):
            field_definitions = []
            for field_definitions_item_data in self._field_definitions:
                if isinstance(field_definitions_item_data, UnknownType):
                    field_definitions_item = field_definitions_item_data.value
                elif isinstance(field_definitions_item_data, SimpleFieldDefinition):
                    field_definitions_item = field_definitions_item_data.to_dict()

                elif isinstance(field_definitions_item_data, IntegerFieldDefinition):
                    field_definitions_item = field_definitions_item_data.to_dict()

                elif isinstance(field_definitions_item_data, FloatFieldDefinition):
                    field_definitions_item = field_definitions_item_data.to_dict()

                elif isinstance(field_definitions_item_data, DropdownFieldDefinition):
                    field_definitions_item = field_definitions_item_data.to_dict()

                else:
                    field_definitions_item = field_definitions_item_data.to_dict()

                field_definitions.append(field_definitions_item)

        id = self._id
        name = self._name

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if type is not UNSET:
            field_dict["type"] = type
        if constraint is not UNSET:
            field_dict["constraint"] = constraint
        if containable_type is not UNSET:
            field_dict["containableType"] = containable_type
        if modified_at is not UNSET:
            field_dict["modifiedAt"] = modified_at
        if prefix is not UNSET:
            field_dict["prefix"] = prefix
        if registry_id is not UNSET:
            field_dict["registryId"] = registry_id
        if archive_record is not UNSET:
            field_dict["archiveRecord"] = archive_record
        if field_definitions is not UNSET:
            field_dict["fieldDefinitions"] = field_definitions
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_type() -> Union[Unset, DeprecatedEntitySchemaType]:
            type = UNSET
            _type = d.pop("type")
            if _type is not None and _type is not UNSET:
                try:
                    type = DeprecatedEntitySchemaType(_type)
                except ValueError:
                    type = DeprecatedEntitySchemaType.of_unknown(_type)

            return type

        try:
            type = get_type()
        except KeyError:
            if strict:
                raise
            type = cast(Union[Unset, DeprecatedEntitySchemaType], UNSET)

        def get_constraint() -> Union[Unset, None, EntitySchemaConstraint]:
            constraint = None
            _constraint = d.pop("constraint")

            if _constraint is not None and not isinstance(_constraint, Unset):
                constraint = EntitySchemaConstraint.from_dict(_constraint)

            return constraint

        try:
            constraint = get_constraint()
        except KeyError:
            if strict:
                raise
            constraint = cast(Union[Unset, None, EntitySchemaConstraint], UNSET)

        def get_containable_type() -> Union[Unset, EntitySchemaContainableType]:
            containable_type = UNSET
            _containable_type = d.pop("containableType")
            if _containable_type is not None and _containable_type is not UNSET:
                try:
                    containable_type = EntitySchemaContainableType(_containable_type)
                except ValueError:
                    containable_type = EntitySchemaContainableType.of_unknown(_containable_type)

            return containable_type

        try:
            containable_type = get_containable_type()
        except KeyError:
            if strict:
                raise
            containable_type = cast(Union[Unset, EntitySchemaContainableType], UNSET)

        def get_modified_at() -> Union[Unset, datetime.datetime]:
            modified_at: Union[Unset, datetime.datetime] = UNSET
            _modified_at = d.pop("modifiedAt")
            if _modified_at is not None and not isinstance(_modified_at, Unset):
                modified_at = isoparse(cast(str, _modified_at))

            return modified_at

        try:
            modified_at = get_modified_at()
        except KeyError:
            if strict:
                raise
            modified_at = cast(Union[Unset, datetime.datetime], UNSET)

        def get_prefix() -> Union[Unset, str]:
            prefix = d.pop("prefix")
            return prefix

        try:
            prefix = get_prefix()
        except KeyError:
            if strict:
                raise
            prefix = cast(Union[Unset, str], UNSET)

        def get_registry_id() -> Union[Unset, str]:
            registry_id = d.pop("registryId")
            return registry_id

        try:
            registry_id = get_registry_id()
        except KeyError:
            if strict:
                raise
            registry_id = cast(Union[Unset, str], UNSET)

        def get_archive_record() -> Union[Unset, None, ArchiveRecord]:
            archive_record = None
            _archive_record = d.pop("archiveRecord")

            if _archive_record is not None and not isinstance(_archive_record, Unset):
                archive_record = ArchiveRecord.from_dict(_archive_record)

            return archive_record

        try:
            archive_record = get_archive_record()
        except KeyError:
            if strict:
                raise
            archive_record = cast(Union[Unset, None, ArchiveRecord], UNSET)

        def get_field_definitions() -> Union[
            Unset,
            List[
                Union[
                    SimpleFieldDefinition,
                    IntegerFieldDefinition,
                    FloatFieldDefinition,
                    DropdownFieldDefinition,
                    SchemaLinkFieldDefinition,
                    UnknownType,
                ]
            ],
        ]:
            field_definitions = []
            _field_definitions = d.pop("fieldDefinitions")
            for field_definitions_item_data in _field_definitions or []:

                def _parse_field_definitions_item(
                    data: Union[Dict[str, Any]]
                ) -> Union[
                    SimpleFieldDefinition,
                    IntegerFieldDefinition,
                    FloatFieldDefinition,
                    DropdownFieldDefinition,
                    SchemaLinkFieldDefinition,
                    UnknownType,
                ]:
                    field_definitions_item: Union[
                        SimpleFieldDefinition,
                        IntegerFieldDefinition,
                        FloatFieldDefinition,
                        DropdownFieldDefinition,
                        SchemaLinkFieldDefinition,
                        UnknownType,
                    ]
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        field_definitions_item = SimpleFieldDefinition.from_dict(data, strict=True)

                        return field_definitions_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        field_definitions_item = IntegerFieldDefinition.from_dict(data, strict=True)

                        return field_definitions_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        field_definitions_item = FloatFieldDefinition.from_dict(data, strict=True)

                        return field_definitions_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        field_definitions_item = DropdownFieldDefinition.from_dict(data, strict=True)

                        return field_definitions_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        field_definitions_item = SchemaLinkFieldDefinition.from_dict(data, strict=True)

                        return field_definitions_item
                    except:  # noqa: E722
                        pass
                    return UnknownType(data)

                field_definitions_item = _parse_field_definitions_item(field_definitions_item_data)

                field_definitions.append(field_definitions_item)

            return field_definitions

        try:
            field_definitions = get_field_definitions()
        except KeyError:
            if strict:
                raise
            field_definitions = cast(
                Union[
                    Unset,
                    List[
                        Union[
                            SimpleFieldDefinition,
                            IntegerFieldDefinition,
                            FloatFieldDefinition,
                            DropdownFieldDefinition,
                            SchemaLinkFieldDefinition,
                            UnknownType,
                        ]
                    ],
                ],
                UNSET,
            )

        def get_id() -> Union[Unset, str]:
            id = d.pop("id")
            return id

        try:
            id = get_id()
        except KeyError:
            if strict:
                raise
            id = cast(Union[Unset, str], UNSET)

        def get_name() -> Union[Unset, str]:
            name = d.pop("name")
            return name

        try:
            name = get_name()
        except KeyError:
            if strict:
                raise
            name = cast(Union[Unset, str], UNSET)

        deprecated_entity_schema = cls(
            type=type,
            constraint=constraint,
            containable_type=containable_type,
            modified_at=modified_at,
            prefix=prefix,
            registry_id=registry_id,
            archive_record=archive_record,
            field_definitions=field_definitions,
            id=id,
            name=name,
        )

        deprecated_entity_schema.additional_properties = d
        return deprecated_entity_schema

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def type(self) -> DeprecatedEntitySchemaType:
        if isinstance(self._type, Unset):
            raise NotPresentError(self, "type")
        return self._type

    @type.setter
    def type(self, value: DeprecatedEntitySchemaType) -> None:
        self._type = value

    @type.deleter
    def type(self) -> None:
        self._type = UNSET

    @property
    def constraint(self) -> Optional[EntitySchemaConstraint]:
        if isinstance(self._constraint, Unset):
            raise NotPresentError(self, "constraint")
        return self._constraint

    @constraint.setter
    def constraint(self, value: Optional[EntitySchemaConstraint]) -> None:
        self._constraint = value

    @constraint.deleter
    def constraint(self) -> None:
        self._constraint = UNSET

    @property
    def containable_type(self) -> EntitySchemaContainableType:
        if isinstance(self._containable_type, Unset):
            raise NotPresentError(self, "containable_type")
        return self._containable_type

    @containable_type.setter
    def containable_type(self, value: EntitySchemaContainableType) -> None:
        self._containable_type = value

    @containable_type.deleter
    def containable_type(self) -> None:
        self._containable_type = UNSET

    @property
    def modified_at(self) -> datetime.datetime:
        """ DateTime the Entity Schema was last modified """
        if isinstance(self._modified_at, Unset):
            raise NotPresentError(self, "modified_at")
        return self._modified_at

    @modified_at.setter
    def modified_at(self, value: datetime.datetime) -> None:
        self._modified_at = value

    @modified_at.deleter
    def modified_at(self) -> None:
        self._modified_at = UNSET

    @property
    def prefix(self) -> str:
        if isinstance(self._prefix, Unset):
            raise NotPresentError(self, "prefix")
        return self._prefix

    @prefix.setter
    def prefix(self, value: str) -> None:
        self._prefix = value

    @prefix.deleter
    def prefix(self) -> None:
        self._prefix = UNSET

    @property
    def registry_id(self) -> str:
        if isinstance(self._registry_id, Unset):
            raise NotPresentError(self, "registry_id")
        return self._registry_id

    @registry_id.setter
    def registry_id(self, value: str) -> None:
        self._registry_id = value

    @registry_id.deleter
    def registry_id(self) -> None:
        self._registry_id = UNSET

    @property
    def archive_record(self) -> Optional[ArchiveRecord]:
        if isinstance(self._archive_record, Unset):
            raise NotPresentError(self, "archive_record")
        return self._archive_record

    @archive_record.setter
    def archive_record(self, value: Optional[ArchiveRecord]) -> None:
        self._archive_record = value

    @archive_record.deleter
    def archive_record(self) -> None:
        self._archive_record = UNSET

    @property
    def field_definitions(
        self,
    ) -> List[
        Union[
            SimpleFieldDefinition,
            IntegerFieldDefinition,
            FloatFieldDefinition,
            DropdownFieldDefinition,
            SchemaLinkFieldDefinition,
            UnknownType,
        ]
    ]:
        if isinstance(self._field_definitions, Unset):
            raise NotPresentError(self, "field_definitions")
        return self._field_definitions

    @field_definitions.setter
    def field_definitions(
        self,
        value: List[
            Union[
                SimpleFieldDefinition,
                IntegerFieldDefinition,
                FloatFieldDefinition,
                DropdownFieldDefinition,
                SchemaLinkFieldDefinition,
                UnknownType,
            ]
        ],
    ) -> None:
        self._field_definitions = value

    @field_definitions.deleter
    def field_definitions(self) -> None:
        self._field_definitions = UNSET

    @property
    def id(self) -> str:
        if isinstance(self._id, Unset):
            raise NotPresentError(self, "id")
        return self._id

    @id.setter
    def id(self, value: str) -> None:
        self._id = value

    @id.deleter
    def id(self) -> None:
        self._id = UNSET

    @property
    def name(self) -> str:
        if isinstance(self._name, Unset):
            raise NotPresentError(self, "name")
        return self._name

    @name.setter
    def name(self, value: str) -> None:
        self._name = value

    @name.deleter
    def name(self) -> None:
        self._name = UNSET



from typing import Union

from ..extensions import UnknownType
from ..models.assay_run_created_event import AssayRunCreatedEvent
from ..models.assay_run_updated_fields_event import AssayRunUpdatedFieldsEvent
from ..models.automation_input_generator_completed_v2_beta_event import (
    AutomationInputGeneratorCompletedV2BetaEvent,
)
from ..models.automation_output_processor_completed_v2_beta_event import (
    AutomationOutputProcessorCompletedV2BetaEvent,
)
from ..models.automation_output_processor_uploaded_v2_beta_event import (
    AutomationOutputProcessorUploadedV2BetaEvent,
)
from ..models.entity_registered_event import EntityRegisteredEvent
from ..models.entry_created_event import EntryCreatedEvent
from ..models.entry_updated_fields_event import EntryUpdatedFieldsEvent
from ..models.entry_updated_review_record_event import EntryUpdatedReviewRecordEvent
from ..models.request_created_event import RequestCreatedEvent
from ..models.request_updated_fields_event import RequestUpdatedFieldsEvent
from ..models.stage_entry_created_event import StageEntryCreatedEvent
from ..models.stage_entry_updated_fields_event import StageEntryUpdatedFieldsEvent
from ..models.stage_entry_updated_review_record_event import StageEntryUpdatedReviewRecordEvent
from ..models.workflow_output_created_event import WorkflowOutputCreatedEvent
from ..models.workflow_output_updated_fields_event import WorkflowOutputUpdatedFieldsEvent
from ..models.workflow_task_created_event import WorkflowTaskCreatedEvent
from ..models.workflow_task_group_created_event import WorkflowTaskGroupCreatedEvent
from ..models.workflow_task_group_updated_watchers_event import WorkflowTaskGroupUpdatedWatchersEvent
from ..models.workflow_task_updated_assignee_event import WorkflowTaskUpdatedAssigneeEvent
from ..models.workflow_task_updated_fields_event import WorkflowTaskUpdatedFieldsEvent
from ..models.workflow_task_updated_scheduled_on_event import WorkflowTaskUpdatedScheduledOnEvent
from ..models.workflow_task_updated_status_event import WorkflowTaskUpdatedStatusEvent

Event = Union[
    EntityRegisteredEvent,
    EntryCreatedEvent,
    EntryUpdatedFieldsEvent,
    EntryUpdatedReviewRecordEvent,
    StageEntryCreatedEvent,
    StageEntryUpdatedFieldsEvent,
    StageEntryUpdatedReviewRecordEvent,
    RequestCreatedEvent,
    RequestUpdatedFieldsEvent,
    AssayRunCreatedEvent,
    AssayRunUpdatedFieldsEvent,
    AutomationInputGeneratorCompletedV2BetaEvent,
    AutomationOutputProcessorCompletedV2BetaEvent,
    AutomationOutputProcessorUploadedV2BetaEvent,
    WorkflowTaskGroupCreatedEvent,
    WorkflowTaskCreatedEvent,
    WorkflowTaskUpdatedFieldsEvent,
    WorkflowTaskUpdatedStatusEvent,
    WorkflowTaskUpdatedAssigneeEvent,
    WorkflowTaskUpdatedScheduledOnEvent,
    WorkflowTaskGroupUpdatedWatchersEvent,
    WorkflowOutputCreatedEvent,
    WorkflowOutputUpdatedFieldsEvent,
    UnknownType,
]



from enum import Enum
from functools import lru_cache
from typing import cast

from ..extensions import Enums


class WorkflowOutputArchiveReason(Enums.KnownString):
    MADE_IN_ERROR = "Made in error"
    RETIRED = "Retired"
    OTHER = "Other"

    def __str__(self) -> str:
        return str(self.value)

    @staticmethod
    @lru_cache(maxsize=None)
    def of_unknown(val: str) -> "WorkflowOutputArchiveReason":
        if not isinstance(val, str):
            raise ValueError(f"Value of WorkflowOutputArchiveReason must be a string (encountered: {val})")
        newcls = Enum("WorkflowOutputArchiveReason", {"_UNKNOWN": val}, type=Enums.UnknownString)  # type: ignore
        return cast(WorkflowOutputArchiveReason, getattr(newcls, "_UNKNOWN"))
